@@ -39,8 +39,8 @@ inline void zaparr(T & x)
      delete [] x;
      x = NULL;
 }
-static char* release_no="0.4alpha";
-static char* release="Freecycle 0.4alpha";
+static char* release_no="0.5alpha";
+static char* release="Freecycle 0.5alpha";
 static const int RTTI_BEATLINE=1001;
 static const int RTTI_BPMLINE=1002;
 static const int RTTI_LOOPMARKER=1003;
@@ -50,6 +50,15 @@ static const int RTTI_DRAWINGLINE=1006;
 static const int RTTI_POLYLINE=1007;
 static const int RTTI_SPLINELINE=1008;
 static const int RTTI_LOCKEDBEATRECT=1009;
+static const int RTTI_LADSPAVALUERECT=1010;
+static const int RTTI_KEYBOARDRECT=1011;
+
+
+static const double BEAT_DISTANCE_TRESHOLD=128;
+
+static const int MIDI_INPUT_EVENT=2000;
+static const int PLAY_POSITION_CHANGED_EVENT=2001;
+
 
 #endif
 
