@@ -88,10 +88,36 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
         scrollTimer = new QTimer( this );
         scrollTimer->stop();
         connect(scrollTimer, SIGNAL(timeout()),this, SLOT(scroll()));
+
+        keyboardView=new KeyboardView(canvas);
+        keyboardView->hide();
+        
+        showInsertAtRect=new ShowInsertAtRect(canvas);
+        showInsertAtRect->hide();
+        
+        viewport()->setAcceptDrops( TRUE );
+        acceptDrop=FALSE;
+        
+
+
+        
+        
 }
 
  void DrawSoundWidget::contentsMousePressEvent( QMouseEvent * mouseEvent){
         QPoint pos=mouseEvent->pos();
+        //remove keyboard view if not clicked
+        if(keyboardView->isVisible() && !keyboardView->rect().contains(pos)){
+            keyboardView->hide();
+            midiEditingEnd();
+            canvas->update();
+        }else if(keyboardView->isVisible() && keyboardView->rect().contains(pos)){
+            keyboardView->mousePressEvent(mouseEvent);
+            canvas->update();
+            return;
+        }
+        
+        
         int pos_x=pos.x();
         long sample_pos=pos.x()*getZoomRatio();
         if(selection->isVisible())
@@ -115,7 +141,8 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
                     loopMarkerR->setMoving(TRUE);
                     canvas->update();
                 }else if(mouseEvent->pos().y()<canvas->getRulerHeight()){
-                    emit(playFromTo(findLeftBeatline(sample_pos),findRightBeatline(sample_pos)));
+                    long temp_pos=mouseEvent->pos().x()*getZoomRatio();
+                    emit(playFromTo(findLeftBeatlinePos(temp_pos),findRightBeatlinePos(temp_pos)));
                 }else if(BeatLine* beatline=getMovableBeatLine(mouseEvent->pos())){
                     beatline->setMoving(TRUE);
                     canvas->setAllChanged();
@@ -127,15 +154,16 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
                     canvas->update();
                 }else if(BeatLine* beatline=getLockableBeatLine(mouseEvent->pos())){
                     if(beatline->locked()){
-                                    beatline->unlock();
-                                    if(beatline->isBpmLine())
-                                            bpmLines.append(beatline);
-                                    else
-                                            beatLines.append(beatline);
-                                    lockedBeatLines.remove(beatline);
-                                    if(lockedBeatLines.count()==0)
-                                        emit(hasLockedBeatLines(FALSE));
-
+                                    if(canUnlock(beatline)){
+                                        beatline->unlock();
+                                        if(beatline->isBpmLine())
+                                                bpmLines.append(beatline);
+                                        else
+                                                beatLines.append(beatline);
+                                        lockedBeatLines.remove(beatline);
+                                        if(lockedBeatLines.count()==0)
+                                            emit(hasLockedBeatLines(FALSE));
+                                    }
                             }else{
                                     beatline->lock();
                                     lockedBeatLines.append(beatline);
@@ -148,11 +176,25 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
                             }
                             updateBeatLines();
                             updateLockedBeatRects();
-                            emit(beatLinesChanged(toArray(beatLines)));
-                            emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
+                            emit(beatLinesChanged(toData(beatLines)));
+                            emit(lockedBeatLinesChanged(toData(lockedBeatLines)));
+                            keyboardView->initKeys(toData(lockedBeatLines));
+                }else if(BeatLine* beatline=getAssignableBeatLine(mouseEvent->pos())){
+                    keyboardView->setX(beatline->x());
+                    keyboardView->setY(canvas->height()-30);
+                    keyboardView->setBeatlinePos(beatline->getBeatPos());
+                    keyboardView->setBeatlineData(toData(lockedBeatLines));
+                    keyboardView->setLoop(loopMarkerL->getSamplePosition(), loopMarkerR->getSamplePosition());
+                    beatline->setEditingMidi(1);
+                    keyboardView->show();
+                    canvas->update();
                 }else if(LockedBeatRect* beatrect=getMovableBeatRect(mouseEvent->pos())){
                     beatrect->setSelected(TRUE);
                     canvas->update();
+                }else if(LockedBeatRect* beatrect=getSNDBeatRect(mouseEvent->pos())){
+                    //beatrect->setSelected(TRUE);
+                    beatrect->mousePressEvent(mouseEvent);
+                    canvas->update();
                 }else{
                     QCanvasItemList colisions=canvas->collisions(pos);
                     /*
@@ -256,7 +298,12 @@ DrawSoundWidget::DrawSoundWidget(QWidget *parent, const char *name,SoundManager*
         handleMouseMoveBeatRect(mouseEvent);
     }
  void DrawSoundWidget::contentsMouseReleaseEvent( QMouseEvent * mouseEvent){
-     scrollTimer->stop();
+    scrollTimer->stop();
+    if(keyboardView->isVisible() && keyboardView->rect().contains(mouseEvent->pos())){
+        keyboardView->mouseReleaseEvent(mouseEvent);
+        canvas->update();
+        return;
+    }
     if (mouseEvent->button()==LeftButton){
         if(mouseEvent->pos().y()<canvas->getRulerHeight()){
             emit(stopPlay());
@@ -315,7 +362,7 @@ void DrawSoundWidget::resized(){
 void DrawSoundWidget::fileLoaded()
 {
     setPositionPointer(0);
-    unlockBeatLines();
+    unlockBeatLines(true);
     removeBeatLines();
     removeBPMLines();
     setBeatLines(soundManager->analyse(16,4));
@@ -330,6 +377,9 @@ void DrawSoundWidget::fileLoaded()
 
 DrawSoundWidget::~DrawSoundWidget()
 {
+
+    //delete scrollTimer; 
+
 }
 
 
@@ -373,7 +423,7 @@ void DrawSoundWidget::removeBeatLines()
                 delete beatline;
         }
         if(updateCanvas) canvas->update();
-        emit(beatLinesChanged(toArray(beatLines)));
+        emit(beatLinesChanged(toData(beatLines)));
 }
 
 void DrawSoundWidget::addBeatLine(long beat){
@@ -395,12 +445,12 @@ void DrawSoundWidget::addBeatLine(long beat){
             beatline->setX((int)((double)beat/getZoomRatio()));
             if(showBeatLinesVal) beatline->show();
             beatLines.append(beatline);
-            emit(beatLinesChanged(toArray(beatLines)));
+            emit(beatLinesChanged(toData(beatLines)));
     }
 
 }
 
-void DrawSoundWidget::addLockedBeatLine(long beat){
+void DrawSoundWidget::addLockedBeatLine(long beat,int midiNote){
     //locks the beatline if already present
     bool alreadyPresent=FALSE;
     QPtrListIterator<BeatLine> it( beatLines );
@@ -420,6 +470,7 @@ void DrawSoundWidget::addLockedBeatLine(long beat){
         beatline->setSampleValueView(sampleValueView);
         beatline->setPen(QColor(50,255,50));
         beatline->setX((int)((double)beat/getZoomRatio()));
+        beatline->setMidiNote(midiNote);
         beatline->lock();
         if(showBeatLinesVal) beatline->show();
         lockedBeatLines.append(beatline);
@@ -441,7 +492,7 @@ void DrawSoundWidget::setBeatLines(QMemArray<long> beats)
                     addBeatLine(beats[i]);
         }
         if(updateCanvas) canvas->update();
-        emit(beatLinesChanged(toArray(beatLines)));
+        emit(beatLinesChanged(toData(beatLines)));
         std::cout<<"beatLines: "<<beatLines.count()<<endl;
         //delete beats;
 }
@@ -517,8 +568,7 @@ void DrawSoundWidget::setTreshold(int _treshold, bool viewOnly)
 {
     treshold=((double)_treshold)/(double)10.0;
     if(viewOnly){
-        setBeatLines(soundManager->analyse(treshold,windowDivider,
-                                                                                getFirstVisibleSample(),getLastVisibleSample()));
+        setBeatLines(soundManager->analyse(treshold,windowDivider,getFirstVisibleSample(),getLastVisibleSample()));
     }else{
         setBeatLines(soundManager->analyse(treshold,windowDivider));
     }
@@ -556,7 +606,7 @@ double DrawSoundWidget::computeBPM2()
 
 
     QProgressDialog progress("Matching BPM...","Abort",
-                                            (BPM_MAX-BPM_MIN),NULL,"progress",TRUE);
+                                            int(BPM_MAX-BPM_MIN),NULL,"progress",TRUE);
     progress.setMinimumDuration(2000);
 
      //if(FALSE){
@@ -585,7 +635,7 @@ double DrawSoundWidget::computeBPM2()
 	double min_distance=DBL_MAX;
 
 	candidate_bpm=0;
-	for (float i=BPM_MIN;i<BPM_MAX;i+=0.010f){
+	for (float i=BPM_MIN;i<BPM_MAX;i+=0.010f){	    
             float beat_distance=(float)(((60.0/(double)i)*soundManager->getRate()));
             float distance_treshold=(float)(beat_distance/64);
             //cout<<beat_distance<<"\n";
@@ -597,7 +647,7 @@ double DrawSoundWidget::computeBPM2()
                 long current_bpm_pos=long((index_min+(k*beat_distance)));
                 //replacement
 
-                long bl_position=findNearestBeatline(current_bpm_pos);
+                long bl_position=findNearestBeatlinePos(current_bpm_pos);
 
                 long distance=abs(current_bpm_pos-bl_position);
                 //std::cout<<"dist: "<<distance<<" dist_tresh: "<<distance_treshold<<endl;
@@ -628,7 +678,7 @@ double DrawSoundWidget::computeBPM2()
 */
             count++;
             if(count%10==0){
-                progress.setProgress((i-BPM_MIN));
+                progress.setProgress(int(i-BPM_MIN));
                 progress.setLabelText("Matching BPM...(BPM: "+QString::number(i)+")");
                 if(progress.wasCanceled()) break;
                 count=0;
@@ -648,14 +698,14 @@ double DrawSoundWidget::computeBPM()
 {
     double candidate_bpm=0;
     int count=0;
-     float BPM_MIN=60.0;
-     float BPM_MAX=200.0;
+     double BPM_MIN=60;
+     double BPM_MAX=200;
      long index_min=LONG_MAX;
      long index_max=LONG_MIN;
 
 
     QProgressDialog progress("Matching BPM...","Abort",
-                                            (BPM_MAX-BPM_MIN),NULL,"progress",TRUE);
+                                            int(BPM_MAX-BPM_MIN),NULL,"progress",TRUE);
     progress.setMinimumDuration(2000);
 
      //if(FALSE){
@@ -683,9 +733,9 @@ double DrawSoundWidget::computeBPM()
 	double min_distance=DBL_MAX;
 
 	candidate_bpm=0;
-	for (float i=BPM_MIN;i<BPM_MAX;i+=0.010f){
-            float beat_distance=(float)(((60.0/(double)i)*soundManager->getRate()))/4;
-            float distance_treshold=(float)(beat_distance/4);
+	for (double i=BPM_MIN;i<BPM_MAX;i+=0.01){
+            double beat_distance=(double)(((60.0/(double)i)*soundManager->getRate()))/4;
+            double distance_treshold=(double)(beat_distance/BEAT_DISTANCE_TRESHOLD);
             //cout<<beat_distance<<"\n";
             matching_beat_lines=0;
             long current_distance=0;
@@ -698,7 +748,7 @@ double DrawSoundWidget::computeBPM()
 
                 long bl_position=beatline->getBeatPos();
                 double current_k=(bl_position-index_min)/beat_distance;
-                float distance=
+                double distance=
                     fabs((current_k-((int)current_k%(int)beat_distance))*beat_distance);
                 //std::cout<<"dist: "<<distance<<" dist_tresh: "<<distance_treshold<<endl;
                 if(distance<distance_treshold){
@@ -791,20 +841,23 @@ void DrawSoundWidget::computeNewBPMLines(const QString & bpm_s){
 /*!
     \fn DrawSoundWidget::unlockBeatLines()
  */
-void DrawSoundWidget::unlockBeatLines()
+void DrawSoundWidget::unlockBeatLines(bool force)
 {
 	QPtrListIterator<BeatLine> it( lockedBeatLines );
 	BeatLine *beatline=0;
 	while ( (beatline = it.current()) != 0 ) {
 		++it;
-		beatline->unlock();
-    		beatLines.append(beatline);
-		lockedBeatLines.remove(beatline);
+		if(force || canUnlock(beatline)){
+                    beatline->unlock();
+                    beatLines.append(beatline);
+                    lockedBeatLines.remove(beatline);
+                }
     	}
     	updateBeatLines();
         updateLockedBeatRects();
-        emit(beatLinesChanged(toArray(beatLines)));
-        emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
+        emit(beatLinesChanged(toData(beatLines)));
+        emit(lockedBeatLinesChanged(toData(lockedBeatLines)));
+        keyboardView->initKeys(toData(lockedBeatLines));
 }
 
 /*!
@@ -822,13 +875,14 @@ void DrawSoundWidget::lockBeatLines()
     }
     updateBeatLines();
     updateLockedBeatRects();
-    emit(beatLinesChanged(toArray(beatLines)));
-    emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
+    emit(beatLinesChanged(toData(beatLines)));
+    emit(lockedBeatLinesChanged(toData(lockedBeatLines)));
+    keyboardView->initKeys(toData(lockedBeatLines));
 }
 
 void DrawSoundWidget::reinitCanvas(){
     //delete Lines
-    unlockBeatLines();
+    unlockBeatLines(true);
     QPtrListIterator<BeatLine> it( beatLines );
     BeatLine *beatline=0;
     while ( (beatline = it.current()) != 0 ) {
@@ -914,11 +968,52 @@ void DrawSoundWidget::resample(){
 	for (uint i=0;i<lockedBeatLines.count();i++){
 			sortedPos.push_front(lockedBeatLines.at(i)->getBeatPos());
 	}
-
 	sortedPos.sort();
 	soundManager->resample(sortedPos,new_to_oldBPM_ratio);
          //zoomAll();  //problem!!
-        reinitCanvas();
+         
+        lockedBeatRects.sort();
+        QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+        LockedBeatRect *beatrect;
+        
+        QPtrListIterator<LockedBeatRect> it2( lockedBeatRects );
+        LockedBeatRect *beatrect2;
+        while ( (beatrect = it.current()) != 0 ) {
+            long leftpos=(*it)->getStartBL()->getBeatPos();
+            BeatLine* rightBL=(*it)->getEndBL();
+            long rightpos=rightBL->getBeatPos();
+            int rightAdvance=((rightpos-leftpos)/new_to_oldBPM_ratio)-(rightpos-leftpos);
+            cout<<"rightAdvance: "<<rightAdvance<<endl;
+            if(it.atFirst() && leftpos>1){
+                int leftAdvance=(leftpos/new_to_oldBPM_ratio)-leftpos;
+                (*it)->getStartBL()->setSamplePos(leftpos+leftAdvance);
+                rightAdvance+=leftAdvance;
+            }
+
+            it2=it;
+            while ( (beatrect2 = it2.current()) != 0 ) {
+                BeatLine* bl2=(*it2)->getEndBL();
+                bl2->setSamplePos(bl2->getBeatPos()+rightAdvance);
+                ++it2;
+            }
+            cout<<"pos left "<<leftpos<<" right "<<rightpos<<"->"<<rightBL->getBeatPos()<<endl;
+            ++it;
+        }
+        
+        removeSelectionRectangles();
+
+        setBeatLines(soundManager->analyse(treshold,windowDivider));
+        setPositionPointer((int)((double)playPosition/getZoomRatio()));
+        updateSelectionRectangles();
+        updateLockedBeatRects();
+        updateBeatLines();
+        updateVerticalSelectionRectangles();
+        updateLoopMarkers();
+        analysisFinished();
+        canvas->update();
+         
+         
+        //reinitCanvas();
 }
 
 
@@ -1105,7 +1200,7 @@ long DrawSoundWidget::getSelectionEndSample()
 
 
 /*!
-    \fn DrawSoundWidget::getLockedBeats()
+    \fn DrawSoundWidget::getBeats()
  */
 void DrawSoundWidget::getBeats(long tempBeats[])
 {
@@ -1130,6 +1225,38 @@ void DrawSoundWidget::getBeats(long tempBeats[])
 
 }
 
+/*!
+    \fn DrawSoundWidget::getLockedMidiNotes()
+ */
+QValueList <int> DrawSoundWidget::getLockedMidiNotes()
+{
+
+    QValueList <int> response;
+     QValueList <beatline_data> lockedBL=getLockedBeatList();
+     for (int i=0;i<lockedBL.count();i++){
+        response.append(lockedBL[i].midiNote);
+     }
+    return response;
+}
+
+/*!
+    \fn DrawSoundWidget::getLockedBeats()
+ */
+void DrawSoundWidget::getLockedBeats(long tempBeats[])
+{
+
+    QPtrListIterator<BeatLine> it( lockedBeatLines );
+    BeatLine *beatline;
+    int count=0;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        tempBeats[count]=beatline->getBeatPos();
+	cout<<"unlocked: "<<tempBeats[count]<<endl;
+	count++;
+    }
+
+}
+
 
 /*!
     \fn DrawSoundWidget::getNbLockedBeats()
@@ -1160,13 +1287,17 @@ void DrawSoundWidget::analysisFinished()
  */
 void DrawSoundWidget::customEvent(QCustomEvent* ev)
 {
-    if(ev->type() == 346798){
+    if(ev->type() == PLAY_POSITION_CHANGED_EVENT){
         if(!((PlayPositionChanged*)ev)->getPlaying()){
             getPositionPointerCI()->setState(PlayLine::STATE_STOPPED);
+            emit(stopPlay());
         }else{
             getPositionPointerCI()->setState(PlayLine::STATE_PLAYING);
         }
         this->setPositionPointer(((PlayPositionChanged*)ev)->getPosition());
+    }else if (ev->type() == MIDI_INPUT_EVENT){
+        MidiInputEvent* midievent=(MidiInputEvent*)ev;
+        midiNoteReceived(midievent->getOnOff(), midievent->getNote(), midievent->getVelocity());
     }
 }
 
@@ -1232,9 +1363,9 @@ void DrawSoundWidget::swapMarkers()
 
 
 /*!
-    \fn DrawSoundWidget::findLeftBeatline(long)
+    \fn DrawSoundWidget::findLeftBeatlinePos(long)
  */
-long DrawSoundWidget::findLeftBeatline(long sample_pos)
+long DrawSoundWidget::findLeftBeatlinePos(long sample_pos)
 {
     int size=getNbBeats();
     long* temp=new long[size];
@@ -1252,9 +1383,9 @@ long DrawSoundWidget::findLeftBeatline(long sample_pos)
 
 
 /*!
-    \fn DrawSoundWidget::findRightBeatline(long)
+    \fn DrawSoundWidget::findRightBeatlinePos(long)
  */
-long DrawSoundWidget::findRightBeatline(long sample_pos)
+long DrawSoundWidget::findRightBeatlinePos(long sample_pos)
 {
     int size=getNbBeats();
     long* temp=new long[size];
@@ -1270,9 +1401,9 @@ long DrawSoundWidget::findRightBeatline(long sample_pos)
     return rightmost;
 }
 
-long DrawSoundWidget::findNearestBeatline(long sample_pos){
-    long left=findLeftBeatline(sample_pos);
-    long right=findRightBeatline(sample_pos);
+long DrawSoundWidget::findNearestBeatlinePos(long sample_pos){
+    long left=findLeftBeatlinePos(sample_pos);
+    long right=findRightBeatlinePos(sample_pos);
     if((sample_pos-left)<(right-sample_pos)) return left;
     else return right;
 }
@@ -1290,7 +1421,7 @@ void DrawSoundWidget::setLoopMarkersOnMeasure(int fst_beatline)
         std::sort(&temp[0],&temp[size-1]);
         loopMarkerL->setSamplePosition(temp[fst_beatline]);
         float mesure_distance=4.0*getBeatDistance();
-        loopMarkerR->setSamplePosition(findNearestBeatline(long(temp[fst_beatline]+mesure_distance)));
+        loopMarkerR->setSamplePosition(findNearestBeatlinePos(long(temp[fst_beatline]+mesure_distance)));
         zaparr(temp);
     }else{
         loopMarkerL->setSamplePosition(0);
@@ -1345,6 +1476,35 @@ void DrawSoundWidget::cropped(long start,long end){
         }
     updateSelectionRectangles();
     updateVerticalSelectionRectangles();
+
+   //remove cropped beatlines
+        BeatLine *beatline;
+        QPtrListIterator<BeatLine> it( beatLines );
+        
+        while ( (beatline = it.current()) != 0 ) {
+                ++it;
+                if(beatline->getBeatPos()<start || beatline->getBeatPos()>end){
+                    beatLines.remove(beatline);
+                    delete beatline;
+                }
+                else{
+                    beatline->setSamplePos(beatline->getBeatPos()-start);
+                }
+        }
+        emit(beatLinesChanged(toData(beatLines)));
+        QPtrListIterator<BeatLine> it2( lockedBeatLines );
+        while ( (beatline = it2.current()) != 0 ) {
+                ++it2;
+                if(beatline->getBeatPos()<start || beatline->getBeatPos()>=end){
+                    lockedBeatLines.remove(beatline);
+                    delete beatline;
+                }else{
+                    beatline->setSamplePos(beatline->getBeatPos()-start);
+                }
+        }
+        canvas->update();
+        emit(lockedBeatLinesChanged(toData(lockedBeatLines)));
+        keyboardView->initKeys(toData(lockedBeatLines));
     zoomAll();
 }
 
@@ -1658,13 +1818,13 @@ void DrawSoundWidget::handleMouseMoveBeatRect(QMouseEvent* mouseEvent)
                                                 .append(soundManager->getFileExtension());
 
         QString orig=tempfilename;
-        orig.prepend("orig-").prepend(soundManager->getTempDir()+"/");
+        orig.prepend(soundManager->getTempDir()+"/").append("-orig");
 
         QString final=tempfilename;
         final.prepend(soundManager->getTempDir()+"/");
 
         QString ladspa=tempfilename;
-        ladspa.prepend("ladspa-").prepend(soundManager->getTempDir()+"/");
+        ladspa.prepend(soundManager->getTempDir()+"/").append("-ladspa");
 
 
         QStrList filelist;
@@ -1672,14 +1832,15 @@ void DrawSoundWidget::handleMouseMoveBeatRect(QMouseEvent* mouseEvent)
         soundManager->saveChunkToAs(   final,
                                        beatrect->getStartSample(),
                                        beatrect->getEndSample(),-1,0,SoundHolder::FOR_OUTPUT);
-        filelist.append(final);
+        filelist.append(QUriDrag::localFileToUri(final));
 
-        /*
+        
         soundManager->saveChunkToAs(   orig,
-                                       beatrect->getStartSample(),
-                                       beatrect->getEndSample(),-1,0,SoundHolder::ORIGINAL_WAVE);
-        filelist.append(orig);
-
+                                       beatrect->getStartSample(),beatrect->getEndSample(),
+                                       soundManager->toBpsCode(32),
+                                       0,SoundHolder::ORIGINAL_WAVE);
+        //filelist.append(orig);
+        /*
         if(soundManager->getHasLADSPA()){
             soundManager->saveChunkToAs(   ladspa,
                                            beatrect->getStartSample(),
@@ -1694,45 +1855,105 @@ void DrawSoundWidget::handleMouseMoveBeatRect(QMouseEvent* mouseEvent)
         QPixmap pixmap2;
         pixmap2.resize((beatrect->getEndSample()-beatrect->getStartSample())/getZoomRatio(),
                                         pixmap.height()-40);
-        copyBlt(&pixmap2,0,0,&pixmap,beatrect->x()-contentsX(),20,pixmap2.width(),pixmap2.height());
+        copyBlt(&pixmap2,0,0,&pixmap,beatrect->x()-contentsX(),
+                20,pixmap2.width(),pixmap2.height());
 
         QWMatrix m;
         m.scale(.5,.25);
         QPixmap scaledPixmap = pixmap2.xForm(m);
         waveDrag->setPixmap(scaledPixmap);
-        waveDrag->dragMove();
+        waveDrag->drag();
         beatrect->setSelected(FALSE);
         canvas->update();
     }
 }
 
-void DrawSoundWidget::contentsDragEnterEvent(QDragEnterEvent* event)
-{
-    cout<<"contentsDragEnterEvent\n";
-    if(lockedBeatLines.count()>0 && QUriDrag::canDecode(event)){
+void DrawSoundWidget::contentsDragEnterEvent(QDragEnterEvent* event){
+    if(lockedBeatLines.count()>0 && QUriDrag::canDecode(event) && lockedBeatLines.count()>=2){
         QStringList list;
+        QString filename;
         QUriDrag::decodeLocalFiles(event,list);
-        cout<<"Dragged file: "<<list.front()<<endl;
-        event->accept(soundManager->canOpen(list.front()));
-    }
+        filename=list.front();
+        if(event->source() == this){
+            filename.append("-orig");
+        }
+        if(!list.empty() && soundManager->canOpen(filename)==TRUE){
+            event->accept();
+            canvas->update();
+        }else event->ignore();
+    }else event->ignore();
+    acceptDrop=event->isAccepted();
 }
 
 void DrawSoundWidget::contentsDragMoveEvent(QDragMoveEvent* event)
 {
-    if(event->isActionAccepted()){
-        QPoint pos=event->pos();
-        QCanvasItemList colisions=canvas->collisions(pos);
-        cout<<"drag pos: "<<pos.x()<<endl;
+    if(acceptDrop){
+        long pos=event->pos().x()*getZoomRatio();
+        BeatLine* left=findLeftLockedBeatLine(pos);
+        BeatLine* right=findRightLockedBeatLine(pos);
+        if(left && right){
+            int pos_x=left->x();
+            int width=right->x()-pos_x;
+            showInsertAtRect->setGeom(pos_x,width);
+            if (!showInsertAtRect->isVisible()) showInsertAtRect->show();
+            canvas->update();
+        }
     }
 }
 
+void DrawSoundWidget::contentsDragLeaveEvent(QDragLeaveEvent* event)
+{
+    showInsertAtRect->hide();
+    canvas->update();
+}
+
 void DrawSoundWidget::contentsDropEvent(QDropEvent* event)
 {
-        int insertpos=event->pos().x();
+     
+     if(acceptDrop){
         QStringList list;
+        QString filename;
         QUriDrag::decodeLocalFiles(event,list);
-        soundManager->insertFileAt(list.front(),0);
-        cout<<"wavefile dropped\n";
+        filename=list.front();
+        if(event->source() == this){
+            filename.append("-orig");
+        }
+        long pos=event->pos().x()*getZoomRatio();
+        BeatLine* left=findLeftLockedBeatLine(pos);
+        BeatLine* right=findRightLockedBeatLine(pos);
+        if(left && right){
+            int pos_x=left->x();
+            QString tempdir=soundManager->getTempDir().append("/");  
+            lockedBeatRects.find(new LockedBeatRect(canvas,left,right,soundManager));
+            QDateTime timeDT=QDateTime::currentDateTime();
+            unsigned long int sec=timeDT.toTime_t();
+            int msec=timeDT.time().msec();
+            QString timeS=QString::number(sec)+QString::number(msec).rightJustify(3,'0');
+            cout<<"time: "<<timeS<<endl;
+            if(lockedBeatRects.current()->getSNDCount()==0){
+                QString replacedfilename=tempdir+timeS;
+                soundManager->saveChunkTo(replacedfilename,left->getBeatPos(),right->getBeatPos(),0);
+                SampleNumberDisplay* replacedSND=new SampleNumberDisplay(canvas);
+                replacedSND->setAssignedFile(replacedfilename);
+                lockedBeatRects.current()->addSampleNumberDisplay(replacedSND);
+                lockedBeatRects.current()->setCurrentSND(replacedSND);
+            }
+            QString newfilename=tempdir+QString::number(sec)+QString::number(msec+1).rightJustify(3,'0');
+            soundManager->replaceFileAt(filename,left->getBeatPos(),right->getBeatPos());
+            soundManager->saveChunkToAs(newfilename,
+                                        left->getBeatPos(),right->getBeatPos(),
+                                        soundManager->toBpsCode(32),0,
+                                        SoundManager::ORIGINAL_WAVE);
+            SampleNumberDisplay* newSND=new SampleNumberDisplay(canvas);
+            newSND->setAssignedFile(newfilename);
+            lockedBeatRects.current()->addSampleNumberDisplay(newSND);
+            lockedBeatRects.current()->setCurrentSND(newSND);
+            
+            acceptDrop=FALSE;
+            showInsertAtRect->hide();
+            canvas->update();
+        }
+     }
 }
 
 
@@ -1835,9 +2056,9 @@ BeatLine* DrawSoundWidget::findNextLockedBeatLine(BeatLine* bl)
 /*!
     \fn DrawSoundWidget::getLockedBeatList()
  */
-QValueList<long> DrawSoundWidget::getLockedBeatList()
+QValueList<beatline_data> DrawSoundWidget::getLockedBeatList()
 {
-    return toArray(lockedBeatLines);
+    return toData(lockedBeatLines);
 }
 
 
@@ -1846,9 +2067,8 @@ QValueList<long> DrawSoundWidget::getLockedBeatList()
  */
 void DrawSoundWidget::updateLockedBeatRects()
 {
-
+    LockedBeatRectList tempLBRL;
     lockedBeatLines.sort();
-    lockedBeatRects.clear();
     QPtrListIterator<BeatLine> it( lockedBeatLines );
     BeatLine *beatline,*beatline2;
     int color=0;
@@ -1857,16 +2077,32 @@ void DrawSoundWidget::updateLockedBeatRects()
         ++it;
         beatline2=it.current();
         if(beatline2!=0){
-            LockedBeatRect* lr=new LockedBeatRect(canvas,
-                                                    beatline,beatline2);
-            lr->setNb(count);
-            lr->setColor(color);
-            lr->show();
-            lockedBeatRects.append(lr);
+            LockedBeatRect* lr=new LockedBeatRect(canvas,beatline,beatline2,soundManager);
+            if(lockedBeatRects.find(lr)!=-1){
+                //cout<<"contains\n";
+                lockedBeatRects.current()->setNb(count);
+                lockedBeatRects.current()->setColor(color);
+                lockedBeatRects.current()->updateGeom();
+                tempLBRL.append(lockedBeatRects.take());
+                delete lr;
+            } else {
+                //cout<<"NOT contains\n";
+                lr->setNb(count);
+                lr->setColor(color);
+                lr->show();
+                tempLBRL.append(lr);
+            }
             color=(color+1)%2;
             count++;
         }
     }
+    lockedBeatRects=tempLBRL;
+    /*
+    LockedBeatRect* lbl;
+    for (lbl = lockedBeatRects.first(); lbl; lbl = lockedBeatRects.next()){
+        cout<<(*lbl).x()<<"-"<<(*lbl).x()+(*lbl).width()<<endl;
+    }
+    */
     if(updateCanvas) canvas->update();
 
 
@@ -1874,19 +2110,22 @@ void DrawSoundWidget::updateLockedBeatRects()
 
 
 /*!
-    \fn DrawSoundWidget::setLockedBeatLines(QValueList <long> lbl)
+    \fn DrawSoundWidget::initLockedBeatLines(QValueList <beatline_data> lbl)
  */
-void DrawSoundWidget::setLockedBeatLines(QValueList <long> lbl)
+void DrawSoundWidget::initLockedBeatLines(QValueList <beatline_data> lbl)
 {
+    removeBeatLines();
     for (int i=0;i<lbl.count();i++){
-        addLockedBeatLine(lbl[i]);
+        addLockedBeatLine(lbl[i].position,lbl[i].midiNote);
     }
+    keyboardView->initKeys(lbl);
     updateCanvas=FALSE;
     updateBeatLines();
     updateLockedBeatRects();
     updateCanvas=TRUE;
     canvas->update();
-    emit(lockedBeatLinesChanged(toArray(lockedBeatLines)));
+    emit(lockedBeatLinesChanged(toData(lockedBeatLines)));
+    emit(midiAssChanged(getLockedBeatList()));
 }
 
 
@@ -1917,3 +2156,298 @@ QValueList <long> DrawSoundWidget::getBeatList()
     qHeapSort(result);
     return result;
 }
+
+
+/*!
+    \fn DrawSoundWidget::getAssignableBeatLine(QPoint pos)
+ */
+BeatLine* DrawSoundWidget::getAssignableBeatLine(QPoint pos)
+{
+    QPtrListIterator<BeatLine> it2( lockedBeatLines );
+    BeatLine *beatline;
+    while ( (beatline = it2.current()) != 0 ) {
+            ++it2;
+            if(beatline->getMidiNoteHandle()->contains(pos)) return beatline;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::midiEditingEnd()
+ */
+void DrawSoundWidget::midiEditingEnd()
+{
+    QPtrListIterator<BeatLine> it2( lockedBeatLines );
+    BeatLine* beatline;
+    while ( (beatline = it2.current()) != 0 ) {
+            ++it2;
+            if(beatline->getEditingMidi()==1){
+                beatline->setEditingMidi(0);
+            }
+            beatline->setMidiNote(keyboardView->getMidiNoteFor(beatline->getBeatPos()));
+    }
+    canvas->setAllChanged();
+    emit(midiAssChanged(getLockedBeatList()));
+}
+
+
+/*!
+    \fn DrawSoundWidget::midiNoteReceived(int onoff, int key, int velocity)
+ */
+void DrawSoundWidget::midiNoteReceived(int onoff, int key, int velocity)
+{
+    int samplePos=keyboardView->getSamplePosFor(key);
+    if(keyboardView->isVisible()){
+        keyboardView->showMidiKey(onoff,key,velocity);
+        canvas->update();
+    }
+}
+
+
+
+
+
+/*!
+    \fn DrawSoundWidget::toData(QPtrList <BeatLine>)
+ */
+QValueList<beatline_data> DrawSoundWidget::toData(QPtrList <BeatLine> beatlines)
+{
+    QValueList<beatline_data> returnArray;
+    QPtrListIterator<BeatLine> it( beatlines );
+    BeatLine *beatline;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        returnArray.append(beatline->getData());
+    }
+    qHeapSort(returnArray);
+    return returnArray;
+}
+
+
+/*!
+    \fn DrawSoundWidget::selectLockedBeatlinesList(long start, long end)
+ */
+QValueList<beatline_data> DrawSoundWidget::selectLockedBeatlinesList(long start, long end)
+{
+    QValueList<beatline_data> returnArray;
+    QPtrListIterator<BeatLine> it( lockedBeatLines );
+    BeatLine *beatline;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        if(beatline->getBeatPos()>=start && beatline->getBeatPos()<end){
+            returnArray.append(beatline->getData());
+        }
+    }
+    qHeapSort(returnArray);
+    if(returnArray.first().position!=start){
+        beatline_data startbl;
+        startbl.locked=1;
+        startbl.midiNote=-1;
+        startbl.position=start;
+        returnArray.prepend(startbl);
+    }
+    if(returnArray.last().position!=end){
+        beatline_data endbl;
+        endbl.locked=1;
+        endbl.midiNote=-1;
+        endbl.position=end;
+        returnArray.prepend(endbl);
+    }
+    return returnArray;
+}
+
+
+/*!
+    \fn DrawSoundWidget::assignMissingMidi(QValueList <beatline_data>)
+ */
+QValueList<beatline_data> DrawSoundWidget::assignMissingMidi(QValueList <beatline_data> bld)
+{
+    QValueList <beatline_data> return_bld=bld;
+    int* lookup=new int[127];
+    
+    for (int i=0;i<127;i++){
+        lookup[i]=keyboardView->getKey(i)==NULL?0:keyboardView->getKey(i)->state==KeyboardView::KEY_ASSIGNED?1:0;
+    }
+    
+    for (int i=0;i<return_bld.count();i++){
+        if(return_bld[i].midiNote==-1){
+            for(int j=0;j<127;j++){
+                if(!lookup[j]){
+                    return_bld[i].midiNote=j;
+                    lookup[j]=1;
+                    break;
+                }
+            }
+        }
+    }
+    
+    zaparr(lookup);
+    return return_bld;
+}
+
+
+/*!
+    \fn DrawSoundWidget::findLeftLockedBeatLine(long pos)
+ */
+BeatLine* DrawSoundWidget::findLeftLockedBeatLine(long pos)
+{
+    lockedBeatLines.sort();
+    QPtrListIterator<BeatLine> it( lockedBeatLines );
+    it.toLast();
+    BeatLine *beatline;
+    while ( (beatline = it.current()) != 0 ) {
+        --it;
+        if (beatline->getBeatPos()<pos) return beatline;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::findRightLockedBeatLine(long pos)
+ */
+BeatLine* DrawSoundWidget::findRightLockedBeatLine(long pos)
+{
+    lockedBeatLines.sort();
+    QPtrListIterator<BeatLine> it( lockedBeatLines );
+    it.toFirst();
+    BeatLine *beatline;
+    while ( (beatline = it.current()) != 0 ) {
+        ++it;
+        if (beatline->getBeatPos()>pos) return beatline;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::getSNDBeatRect(QPoint pos)
+ */
+LockedBeatRect* DrawSoundWidget::getSNDBeatRect(QPoint pos)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        if(beatrect->getSampleNumberDisplaysRect().contains(pos)) return beatrect;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::sceneSwitched(int sc)
+ */
+void DrawSoundWidget::sceneSwitched(int sc)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        beatrect->switchScene(sc);
+    }
+}
+
+
+/*!
+    \fn DrawSoundWidget::assignToScene(int sc)
+ */
+void DrawSoundWidget::assignToScene(int sc)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        beatrect->assignSceneToCurrent(sc);
+    }
+}
+
+
+/*!
+    \fn DrawSoundWidget::getLockedBeatRects()
+ */
+LockedBeatRectList DrawSoundWidget::getLockedBeatRects()
+{
+    return lockedBeatRects;
+}
+
+
+/*!
+    \fn DrawSoundWidget::addStackElement(long start, long end, QString filename, long index, int scene, bool current)
+ */
+void DrawSoundWidget::addStackElement(long start, long end, QString filename, long index, int scene, bool current)
+{
+    LockedBeatRect* lbr=getLockedBeatRectFromTo(start,end);
+    if(!lbr) return;
+    SampleNumberDisplay* snd=new SampleNumberDisplay(canvas);
+    snd->setAssignedFile(filename);
+    snd->setNumber(index);
+    snd->setScene(scene);
+    snd->setCurrent(current);
+    lbr->addSampleNumberDisplay(snd);
+    lbr->updateSceneStatus();
+    
+}
+
+
+/*!
+    \fn DrawSoundWidget::getLockedBeatRectFromTo(long start, long end)
+ */
+LockedBeatRect* DrawSoundWidget::getLockedBeatRectFromTo(long start, long end)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        if(beatrect->getStartSample()==start && beatrect->getEndSample()==end)
+            return beatrect;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::canUnlock(BeatLine* bl)
+ */
+bool DrawSoundWidget::canUnlock(BeatLine* bl)
+{
+    LockedBeatRect* right=getLockedBeatRectFrom(bl->getBeatPos());
+    LockedBeatRect* left=getLockedBeatRectTo(bl->getBeatPos());
+    bool canRemoveLeft=(!left || left->canRemove());
+    bool canRemoveRight=(!right || right->canRemove());
+    //if(!canRemoveLeft) left->startBlink(left,2,50,2000);
+    //if(!canRemoveRight)right->startBlink(right,2,50,2000);
+    return canRemoveRight && canRemoveLeft;
+}
+
+
+/*!
+    \fn DrawSoundWidget::getLockedBeatRectFrom(long left)
+ */
+LockedBeatRect* DrawSoundWidget::getLockedBeatRectFrom(long left)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        if(beatrect->getStartSample()==left) return beatrect;
+    }
+    return 0;
+}
+
+
+/*!
+    \fn DrawSoundWidget::getLockedBeatRectTo(long right)
+ */
+LockedBeatRect* DrawSoundWidget::getLockedBeatRectTo(long right)
+{
+    QPtrListIterator<LockedBeatRect> it( lockedBeatRects );
+    LockedBeatRect *beatrect;
+    while ( (beatrect = it.current()) != 0 ) {
+        ++it;
+        if(beatrect->getEndSample()==right) return beatrect;
+    }
+    return 0;
+}
+
