@@ -48,6 +48,11 @@
 #include "lockedbeatrect.h"
 #include "beatlinelist.h"
 #include "wavedrag.h"
+#include "keyboardview.h"
+#include "showinsertatrect.h"
+#include "lockedbeatrectlist.h"
+#include "midiinputevent.h"
+
 /**
 @author Predrag Viceic
 */
@@ -78,6 +83,8 @@ public:
         long getSelectionEndSample();
         int getNbLockedBeats();
         void getBeats(long[]);
+        void getLockedBeats(long[]);
+        QValueList <int> getLockedMidiNotes();
         int getNbBeats();
         void customEvent(QCustomEvent*);
         double getComputedBPM();
@@ -86,9 +93,9 @@ public:
         long getLeftLoopMarkerSample();
         long getRightLoopMarkerSample();
         void swapMarkers();
-        long findLeftBeatline(long);
-        long findRightBeatline(long);
-        long findNearestBeatline(long);
+        long findLeftBeatlinePos(long);
+        long findRightBeatlinePos(long);
+        long findNearestBeatlinePos(long);
         void setLoopMarkersOnMeasure(int);
     float getBeatDistance();
     void setComputedBPMModifier(double);
@@ -101,16 +108,28 @@ public:
     BeatLine* getLockableBeatLine(QPoint pos);
     BeatLine* getRemovableBeatLine(QPoint pos);
     BeatLine* getMovingBeatLine();
+    BeatLine* getAssignableBeatLine(QPoint pos);
     LockedBeatRect* getMovableBeatRect(QPoint pos);
     LockedBeatRect* getMovingBeatRect();
     void handleMouseMoveBeatLine(QMouseEvent* mouseEvent);
     void checkAndScroll(QMouseEvent* ev);
     QValueList<long> toArray(QPtrList<BeatLine> beatlines);
     BeatLine* findNextLockedBeatLine(BeatLine* bl);
-    QValueList<long> getLockedBeatList();
+    QValueList<beatline_data> getLockedBeatList();
     LoopMarker* getLeftLoopMarker();
     LoopMarker* getRightLoopMarker();
+    KeyboardView* keyboardView;
     QValueList <long> getBeatList();
+    QValueList<beatline_data> toData(QPtrList <BeatLine>);
+    QValueList<beatline_data> selectLockedBeatlinesList(long start, long end);
+    QValueList<beatline_data> assignMissingMidi(QValueList <beatline_data>);
+    BeatLine* findLeftLockedBeatLine(long pos);
+    BeatLine* findRightLockedBeatLine(long pos);
+    LockedBeatRectList getLockedBeatRects();
+    LockedBeatRect* getLockedBeatRectFromTo(long start, long end);
+    bool canUnlock(BeatLine* bl);
+    LockedBeatRect* getLockedBeatRectFrom(long left);
+    LockedBeatRect* getLockedBeatRectTo(long right);
 
 
 
@@ -118,14 +137,17 @@ public:
         void contentsMousePressEvent( QMouseEvent * );
         void contentsMouseReleaseEvent( QMouseEvent * );
         void contentsMouseMoveEvent( QMouseEvent * );
-        virtual void contentsDragEnterEvent(QDragEnterEvent* event);
-        virtual void contentsDragMoveEvent(QDragMoveEvent* event);
-        virtual void contentsDropEvent(QDropEvent* event);
+        void contentsDragEnterEvent(QDragEnterEvent* event);
+        void contentsDragMoveEvent(QDragMoveEvent* event);
+        void contentsDragLeaveEvent ( QDragLeaveEvent * );
+        void contentsDropEvent(QDropEvent* event);
+        void midiNoteReceived(int onnoff,int key, int velocity); 
+        bool acceptDrop;
 
     public slots:
         void computeNewBPMLines(const QString &);
         void resample();
-        void unlockBeatLines();
+        void unlockBeatLines(bool force=FALSE);
         void lockBeatLines();
         void setBPMLines(bool on);
         void removeBPMLines();
@@ -137,12 +159,15 @@ public:
         void analysisFinished();
         void setPositionPointer(long);
         void addBeatLine(long);
-        void addLockedBeatLine(long beat);
+        void addLockedBeatLine(long beat,int midiNote=-1);
         void cropped(long,long);
         void showBeatLines(bool);
         void scroll();
         void updateLockedBeatRects();
-        void setLockedBeatLines(QValueList <long> lbl);
+        void initLockedBeatLines(QValueList <beatline_data> lbl);
+        void assignToScene(int sc);
+        
+        
 
 
 
@@ -155,14 +180,17 @@ public:
         void stopPlay();
         void loopChange();
         void hasLockedBeatLines(bool);
-        void receivedEnvelope(QPoint,QString);
-        void beatLinesChanged(QValueList<long> beatLines);
-        void lockedBeatLinesChanged(QValueList<long> beatLines);
+        void receivedWave(QPoint,QString);
+        void beatLinesChanged(QValueList<beatline_data> beatLines);
+        void lockedBeatLinesChanged(QValueList<beatline_data> beatLines);
+        void midiAssChanged(QValueList<beatline_data> bld);
 
 
 private:
         QPixmap* soundpixmap;
         void zoomChanged();
+        void midiEditingEnd();
+    LockedBeatRect* getSNDBeatRect(QPoint pos);
         float savedRelativeSamplePos;
         QPoint scrollPos;
         QMouseEvent* storedMouse;
@@ -174,7 +202,7 @@ protected:
         BeatLineList beatLines;
         BeatLineList lockedBeatLines;
         BeatLineList newBeatLines;
-        QPtrList<LockedBeatRect> lockedBeatRects;
+        LockedBeatRectList lockedBeatRects;
         SelectionRectangle* selection;
         QPtrList<VerticalSelectionRectangle>verticalSelections; // array of VerticalSelectionRectangles
         LoopMarker* loopMarkerL;
@@ -188,9 +216,13 @@ protected:
         bool showBeatLinesVal;
         float computedBPMModifier;
         SampleValueView* sampleValueView;
-        QTimer* fastBlinker;
         QTimer* scrollTimer;
+        ShowInsertAtRect* showInsertAtRect;
+        
 
+protected slots:
+    void sceneSwitched(int sc);
+    void addStackElement(long start, long end, QString filename, long index, int scene, bool current);
 };
 
 #endif
