@@ -20,7 +20,7 @@
 #include "drawsoundwidgetabstract.h"
 
 DrawSoundWidgetAbstract::DrawSoundWidgetAbstract(QWidget *parent, const char *name, SoundManager* sm)
- : QCanvasView(parent,name,WNoAutoErase)
+    : QCanvasView(parent,name,Qt::WNoAutoErase)
 {
     soundManager=sm;
     canvas=new WaveCanvas(this->viewport());
@@ -160,7 +160,7 @@ long DrawSoundWidgetAbstract::checkSamplePosition(long sample_pos)
 */
 void DrawSoundWidgetAbstract::setZoomRatio(long _zr,bool repaint){
     cout<<"in "<<name()<<" setZoomRatio()\n";
-    zoomRatio=_zr;
+    zoomRatio=_zr!=0?_zr:1;
     canvas->setZoomRatio(zoomRatio);
     //on init
     int c_height=
