@@ -20,7 +20,7 @@
 #include "effectportframe.h"
 
 EffectPortFrame::EffectPortFrame(QString _name,LADSPAPort* _port,QWidget *parent, const char *name)
- : QCanvasView(parent, name)
+    : QCanvasView(parent, name,Qt::WNoAutoErase)
 {
     enableClipper(TRUE);
     zoomRatio=1;
@@ -30,7 +30,11 @@ EffectPortFrame::EffectPortFrame(QString _name,LADSPAPort* _port,QWidget *parent
     linesCreated=FALSE;
     //setGeometry(((QVBox*)parent)->contentsRect());
     canvas=new ControllerCanvas(viewport());
-    canvas->setBackgroundColor(lightGray);
+    canvas->setBackgroundColor(QColor(230,230,230));
+    canvas->setMaxValue(port->getUpperBound());
+    canvas->setMinValue(port->getLowerBound());
+    canvas->setStartX(contentsX());
+    canvas->setVisibleWidth(visibleWidth());
     setResizePolicy(QScrollView::AutoOneFit);
     setCanvas(canvas);
     drawValueLines();
@@ -50,12 +54,20 @@ EffectPortFrame::EffectPortFrame(QString _name,LADSPAPort* _port,QWidget *parent
     spline->setMaxValue(port->getUpperBound());
     spline->setMinValue(port->getLowerBound());
     spline->updateSize(canvas->size());
+    ladspaValueView=new LADSPAValueView(canvas);
+    ladspaValueView->hide();
+
+    connect(this,SIGNAL(contentsMoving(int,int)), this,SLOT(contentsMoved(int,int)));
 }
 
 
 EffectPortFrame::~EffectPortFrame()
 {
-
+    delete ladspaValueView;    
+    delete drawingLine;
+    delete polyLine;
+    delete spline;
+    delete canvas;
 }
 
 void EffectPortFrame::raiseMe(){
@@ -79,10 +91,13 @@ void EffectPortFrame::resizeEvent(QResizeEvent* ev)
     QCanvasView::resizeEvent(ev);
     canvas->resize((int)((double)port->getNbFrames()/(double)zoomRatio),
                     visibleHeight());
+    canvas->setStartX(contentsX());
+    canvas->setVisibleWidth(visibleWidth());
     drawValueLines();
     drawBeatlines();
     polyLine->updateSize(canvas->size());
     spline->updateSize(canvas->size());
+
     canvas->update();
     //updateGeometry();
 }
@@ -91,13 +106,14 @@ void EffectPortFrame::drawValueLines(){
     float max=port->getUpperBound();
     float min=port->getLowerBound();
     float diff=max-min;
-
+    int count=0;
     if(!linesCreated){
         for (double i=0;i<port->getNbFrames();i+=1024){
                 ValueLine* line=new ValueLine(canvas,(long)i,(float*)port->getVOTPointer((int)i/1024));
                 int drawValue=(int)(visibleHeight()-((line->getValue()-min)*visibleHeight())/diff);
                 line->setPen(darkGray);
                 line->setPoints((int)(i/zoomRatio),drawValue,(int)(i/zoomRatio),visibleHeight());
+                line->setWidth(ceil(1024/zoomRatio));
                 line->show();
         }
         linesCreated=TRUE;
@@ -105,6 +121,7 @@ void EffectPortFrame::drawValueLines(){
         QCanvasItemList l=canvas->allItems();
         for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
             if((*it)->rtti()==RTTI_VALUELINE){
+                count++;
                 ValueLine* templine=(ValueLine*)(*it);
                 if (templine->isVisible()){
                     int drawValue=
@@ -113,26 +130,15 @@ void EffectPortFrame::drawValueLines(){
                                                         drawValue,
                                                     templine->getSamplePos()/zoomRatio,
                                                     visibleHeight());
+                    templine->setWidth(ceil(1024/zoomRatio));
                 }
             }
         }
     }
     canvas->update();
+    //cout<<"VALUELINES_NB= "<<count<<endl;
 }
 
-/*!
-    \fn EffectPortFrame::repaintEvent(QPaintEvent*)
- */
-void EffectPortFrame::paintEvent(QPaintEvent*)
-{
-    //canvas->resize(visibleWidth(),visibleHeight());
-    //drawValueLines();
-    /*
-    canvas->resize((int)(port->getNbFrames()/zoomRatio),
-                                            c_height);
-    */
-    //canvas->update();
-}
 
 void EffectPortFrame::closeEvent( QCloseEvent* ce ){
 
@@ -179,6 +185,10 @@ ControllerCanvas* EffectPortFrame::getCanvas()
 
 void EffectPortFrame::contentsMousePressEvent( QMouseEvent * mouseEvent){
     QPoint pos=mouseEvent->pos();
+    ladspaValueView->setX(pos.x()+10);
+    ladspaValueView->setY(pos.y());
+    ladspaValueView->setValue(yPosToVal(pos.y()));
+    ladspaValueView->show();
     pos.setX(pos.x()<contentsX()?
             contentsX():pos.x()>contentsX()+width()?contentsX()+width():pos.x());
     //magnet selection
@@ -245,6 +255,9 @@ void EffectPortFrame::contentsMousePressEvent( QMouseEvent * mouseEvent){
 
 void EffectPortFrame::contentsMouseMoveEvent( QMouseEvent * mouseEvent){
     QPoint pos=mouseEvent->pos();
+    ladspaValueView->setX(pos.x()+10);
+    ladspaValueView->setY(pos.y());
+    ladspaValueView->setValue(yPosToVal(pos.y()));
     pos.setX(pos.x()<contentsX()?
             contentsX():pos.x()>contentsX()+width()?contentsX()+width():pos.x());
     pos.setY(pos.y()<0?
@@ -291,6 +304,8 @@ void EffectPortFrame::contentsMouseMoveEvent( QMouseEvent * mouseEvent){
 }
 
 void EffectPortFrame::contentsMouseReleaseEvent( QMouseEvent * mouseEvent){
+    ladspaValueView->hide();
+    canvas->update();
     QPoint pos=mouseEvent->pos();
     pos.setX(pos.x()<contentsX()?
             contentsX():pos.x()>contentsX()+width()?contentsX()+width():pos.x());
@@ -307,6 +322,7 @@ void EffectPortFrame::contentsMouseReleaseEvent( QMouseEvent * mouseEvent){
                                 int y=drawingLine->getYAtX(temp->startPoint().x());
                                 temp->setPoints(temp->startPoint().x(),y,
                                                                 temp->endPoint().x(),visibleHeight());
+                                temp->setWidth(ceil(1024/zoomRatio));
                                 if (temp->getSamplePos()<update_interval.start_sample) {
                                     update_interval.start_sample=temp->getSamplePos();
                                 }
@@ -355,6 +371,7 @@ void EffectPortFrame::contentsMouseReleaseEvent( QMouseEvent * mouseEvent){
 
                             *(temp->getValuePointer())=controlerValue;
                             temp->setPoints(temp->startPoint().x(),y,temp->endPoint().x(),visibleHeight());
+                            temp->setWidth(ceil(1024/zoomRatio));
                             if (temp->getSamplePos()<update_interval.start_sample) {
                                 update_interval.start_sample=temp->getSamplePos();
                             }
@@ -420,6 +437,7 @@ EffectPortFrame::interval EffectPortFrame::updateValueAtPos(QPoint pos,bool left
                         ValueLine* temp=(ValueLine*)(*it);
                         *(temp->getValuePointer())=controlerValue;
                         temp->setPoints(temp->startPoint().x(),y,temp->endPoint().x(),visibleHeight());
+                        temp->setWidth(ceil(1024/zoomRatio));
                         if (temp->getSamplePos()<intdown) {intdown=temp->getSamplePos();}
                         if (temp->getSamplePos()>intup) {intup=temp->getSamplePos();}
                     }
@@ -516,9 +534,9 @@ EffectPortFrame::DrawMode EffectPortFrame::getDrawMode()
 
 
 /*!
-    \fn EffectPortFrame::beatLinesChanged(QValueList <long> beatlines)
+    \fn EffectPortFrame::beatLinesChanged(QValueList <beatline_data> beatlines)
  */
-void EffectPortFrame::beatLinesChanged(QValueList <long> _beatlines)
+void EffectPortFrame::beatLinesChanged(QValueList <beatline_data> _beatlines)
 {
     beatlines=_beatlines;
     drawBeatlines();
@@ -527,9 +545,9 @@ void EffectPortFrame::beatLinesChanged(QValueList <long> _beatlines)
 
 
 /*!
-    \fn EffectPortFrame::lockedBeatLinesChanged(QValueList <long> beatlines)
+    \fn EffectPortFrame::lockedBeatLinesChanged(QValueList <beatline_data> beatlines)
  */
-void EffectPortFrame::lockedBeatLinesChanged(QValueList <long> _beatlines)
+void EffectPortFrame::lockedBeatLinesChanged(QValueList <beatline_data> _beatlines)
 {
     lockedBeatlines=_beatlines;
     drawBeatlines();
@@ -557,13 +575,13 @@ void EffectPortFrame::drawBeatlines()
     cout<<"Drawing "<<lockedBeatlines.count()<<" locked beatlines\n";
     if(lockedBeatlines.count()!=0){
         for (int i=0;i<lockedBeatlines.count();i++){
-            BeatLine* line=new BeatLine(canvas,lockedBeatlines[i]);
+            BeatLine* line=new BeatLine(canvas,lockedBeatlines[i].position);
             line->lock();
             line->setZ(10);
             line->setDrawHandles(0);
             line->setX(line->getBeatPos() / zoomRatio);
             line->setPen(green);
-            int x=lockedBeatlines[i]/zoomRatio;
+            int x=lockedBeatlines[i].position/zoomRatio;
             line->setPoints(x,0,x,visibleHeight());
             line->show();
         }
@@ -723,3 +741,44 @@ void EffectPortFrame::reinitLADSPAValues()
     canvas->setAllChanged();
     canvas->update();
 }
+
+
+
+
+/*!
+    \fn EffectPortFrame::contentsMoved(int x, int y)
+ */
+void EffectPortFrame::contentsMoved(int x, int y)
+{
+    canvas->setStartX(x);
+    canvas->setVisibleWidth(visibleWidth());
+    canvas->setAllChanged();
+    canvas->update();
+}
+
+
+/*!
+    \fn EffectPortFrame::crop(long start, long end)
+ */
+void EffectPortFrame::crop(long start, long end)
+{
+    cout<<"cropping port frame\n";
+    if (hasPolyline()) polyLine->crop(start,end);
+    if (hasSpline()) spline->crop(start,end);
+    long rest=1024-(start%1024);
+    port->crop(start,end);
+    QCanvasItemList l=canvas->allItems();
+    for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
+            if((*it)->rtti()==RTTI_VALUELINE){
+                ValueLine* templine=(ValueLine*)(*it);
+                if(templine->getSamplePos()<start || templine->getSamplePos()>=end){
+                    templine->hide();
+                    delete templine;
+                }else{
+                    templine->setSamplePos(templine->getSamplePos()-start-rest);
+                    templine->setValuePointer(port->getVOTPointer((int)(templine->getSamplePos()/1024)));
+                }
+            }
+    }
+    drawValueLines();
+}
