@@ -34,6 +34,8 @@
 #include "splineline.h"
 #include "defs.h"
 #include "beatline.h"
+#include "ladspavalueview.h"
+#include <qpainter.h>
 /**
 @author Predrag Viceic
 */
@@ -111,8 +113,13 @@ protected:
     DrawingLine* drawingLine;
     PolyLine* polyLine;
     SplineLine* spline;
-    QValueList <long> beatlines;
-    QValueList <long> lockedBeatlines;
+    QValueList <beatline_data> beatlines;
+    QValueList <beatline_data> lockedBeatlines;
+    LADSPAValueView* ladspaValueView;
+
+    void resizeEvent(QResizeEvent*);
+    void closeEvent( QCloseEvent*);
+    float yPosToVal(int y);
 
 
 public slots:
@@ -122,17 +129,13 @@ public slots:
     void linesToggled();
     void polylinesToggled();
     void splinesToggled();
-    void beatLinesChanged(QValueList <long> beatlines);
-    void lockedBeatLinesChanged(QValueList <long> beatlines);
+    void beatLinesChanged(QValueList <beatline_data> beatlines);
+    void lockedBeatLinesChanged(QValueList <beatline_data> beatlines);
     void reinitLADSPAValues();
+    void contentsMoved(int x, int y);
+    void crop(long start, long end);
 
 
-protected:
-    void paintEvent(QPaintEvent*);
-    void resizeEvent(QResizeEvent*);
-    void closeEvent( QCloseEvent*);
-    float yPosToVal(int y);
-
 signals:
     void valueChanged(long,long);
     void drawModeChanged(EffectPortFrame::DrawMode dm);
