@@ -62,6 +62,7 @@ Freecycle::Freecycle()
 {
     QDir freecycle_dir(QDir::homeDirPath().append("/.freecycle"));
     if (!freecycle_dir.exists()) freecycle_dir.mkdir(freecycle_dir.absPath());
+    helpManager=new HelpManager(freecycle_dir.path());
 
     soundManager=new SoundManager();
     soundManager->loadEffects();
@@ -69,7 +70,8 @@ Freecycle::Freecycle()
     soundManager->setFileManager(fileManager);
     midiManager=new MidiManager();
     akpManager=new AKPManager();
-
+    sf2Manager=new SF2Manager();
+    gigaManager=new GigaManager();
     QPixmap openIcon, saveIcon, printIcon;
     openIcon = QPixmap( fileopen );
     saveIcon = QPixmap( filesave );
@@ -80,13 +82,17 @@ Freecycle::Freecycle()
     filePopupMenu->insertItem( openIcon, tr("&Open Wave..."),
 			   this, SLOT(choose()), CTRL+Key_O );
 
-    idsavedialog=filePopupMenu->insertItem( saveIcon, tr("&Save Wave"),
-			   this, SLOT(save()), CTRL+Key_S );
+    idsavedialog=filePopupMenu->insertItem( saveIcon, tr("Save Wave"),
+			   this, SLOT(save()));
 
     idsaveasdialog= filePopupMenu->insertItem( tr("Save Wave &As..."), this, SLOT(saveAs()) );
     idsavechunksdialog=filePopupMenu->insertItem( tr("Save Audio Chunks..."), this, SLOT(saveChunks()) );
-    idsaveprojectdialog=filePopupMenu->insertItem( tr("Save Project..."), this, SLOT(saveProject()) );
+    idsaveprojectasdialog=filePopupMenu->insertItem( tr("Save Project As..."), this, SLOT(saveProjectAs()) );
+    idsaveprojectdialog=filePopupMenu->insertItem( tr("&Save Project"), this, SLOT(saveProject()),CTRL+Key_S );
+    idsaveloopmidifile=filePopupMenu->insertItem( tr("Export Loop &MIDI file..."), this, 
+                                                    SLOT(saveLoopMidiFile()),CTRL+Key_M );
     idloadprojectdialog=filePopupMenu->insertItem( tr("Load Project..."), this, SLOT(loadProject()) );
+    
 
 
     filePopupMenu->insertSeparator();
@@ -146,6 +152,9 @@ Freecycle::Freecycle()
     configurationPopupMenu->insertItem(tr("Configure FFT ..."),this,SLOT(configureFFT()));
     ampDomainDialog=new AmpDomainDialog(soundManager,this);
     configurationPopupMenu->insertItem(tr("Configure Amplitude domain ..."),this,SLOT(configureAmpDomain()));
+    midiInput=new MidiInput();
+    midiConfigDialog=new MidiConfigDialog(midiInput,this);
+    configurationPopupMenu->insertItem(tr("Configure MIDI ..."),this,SLOT(configureMIDI()));
 
 
     menuBar()->insertItem( tr("&Configuration"), configurationPopupMenu,3);
@@ -166,7 +175,7 @@ Freecycle::Freecycle()
 
     //Top status bar
     topBox=new QHBox(centralLayout,"File info");
-    topBox->setSpacing(10);
+    topBox->setSpacing(5);
     topBox->setMaximumHeight(50);
     topBox->setFrameStyle(QFrame::Panel | QFrame::Raised);
     topBox->hide();
@@ -192,7 +201,7 @@ Freecycle::Freecycle()
 
     //End Top status Bar
     playAndBeatLinesControls=new QHBox(centralLayout,"Play and BeatLines");
-    playAndBeatLinesControls->setSpacing(10);
+    playAndBeatLinesControls->setSpacing(0);
     playAndBeatLinesControls->setSizePolicy(QSizePolicy::Maximum,QSizePolicy::Minimum);
     playAndBeatLinesControls->hide();
     //Play buttons bar
@@ -287,6 +296,13 @@ Freecycle::Freecycle()
     connect(envelopeControllerWidget,SIGNAL(envelopeChanged(EnvelopeData* )),
             soundManager,SLOT(envelopeChanged(EnvelopeData* )));
 
+    sceneFrame=new SceneFrame(playAndBeatLinesControls);
+    sceneFrame->setFixedHeight(50);
+    sceneFrame->setFixedWidth(182);
+    sceneFrame->setFrameStyle(QFrame::Panel | QFrame::Raised);
+    sceneFrame->setSpacing(0);
+    sceneFrame->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Fixed);
+    
 
     big_and_small_wave_and_effects=new MySplitter(Qt::Vertical,centralLayout);
     //big_and_small_wave_and_effects->setOpaqueResize(TRUE);
@@ -301,27 +317,33 @@ Freecycle::Freecycle()
 
     drawSoundWidget=new DrawSoundWidget(big_wave_and_env,"central wave",soundManager);
     drawSoundWidget->enableClipper(TRUE);
+    connect(sceneFrame,SIGNAL(sceneSwitchedS(int)),drawSoundWidget,SLOT(sceneSwitched(int )));
+    connect(sceneFrame,SIGNAL(assignToSceneS(int)),drawSoundWidget,SLOT(assignToScene(int )));
 
     bpmLines=new QPushButton("bpm lines",topBox);
     bpmLines->setToggleButton(TRUE);
     bpmLines->setPixmap(QPixmap(qembed_findImage("kalarm")));
     bpmLines->setDisabled(TRUE);
     connect(drawSoundWidget,SIGNAL(hasLockedBeatLines(bool)),bpmLines,SLOT(setEnabled(bool)));
+    
+    midiInput->addMidiMonitor(drawSoundWidget);
 
-    envelopeHolder=new EnvelopeHolder(big_wave_and_env);
-    envelopeHolder->setMaximumHeight(55);
-    envelopeHolder->setMinimumHeight(55);
-    envelopeHolder->hide();
+    //replacedWaveHolder=new ReplacedWaveHolder(big_wave_and_env);
+    //replacedWaveHolder->setMaximumHeight(55);
+    //replacedWaveHolder->setMinimumHeight(55);
+    //replacedWaveHolder->hide();
+    /*
     connect(drawSoundWidget,
             SIGNAL(contentsMoving(int,int)),
-            envelopeHolder,SLOT(setContentsPos(int,int)));
+            replacedWaveHolder,SLOT(setContentsPos(int,int)));
     connect(drawSoundWidget,
-            SIGNAL(receivedEnvelope(QPoint,QString)),
-            envelopeHolder,SLOT(setEnvelopeAt(QPoint,QString)));
+            SIGNAL(receivedWave(QPoint,QString)),
+            replacedWaveHolder,SLOT(setWaveAt(QPoint,QString)));
     connect(drawSoundWidget,SIGNAL(zoomChangedS(long)),
-            envelopeHolder,SLOT(setZoomRatio(long)));
+            replacedWaveHolder,SLOT(setZoomRatio(long)));
     connect(soundManager,SIGNAL(nbFramesChanged(long)),
-            envelopeHolder,SLOT(setFrames(long)));
+            replacedWaveHolder,SLOT(setFrames(long)));
+    */
     //envelopeHolder->setHeight(0);
 
     zoomBox=new QHBox(big_wave_and_zoom);
@@ -400,6 +422,9 @@ Freecycle::Freecycle()
     connect(drawSoundWidget,SIGNAL(playFromPoint(long)),this,SLOT(playFromPoint(long)));
     connect(drawSoundWidget,SIGNAL(playFromTo(long,long)),this,SLOT(playFromTo(long,long)));
     connect(drawSoundWidget,SIGNAL(stopPlay()),this,SLOT(stopPlay()));
+    
+    connect(drawSoundWidget,SIGNAL(midiAssChanged(QValueList <beatline_data>)),
+            midiInput, SLOT(setBeatlineData(QValueList <beatline_data>)));
 
 
     spectrumView=new SpectrumView(small_waveAndSpectrum);
@@ -416,6 +441,7 @@ Freecycle::Freecycle()
     soundPlayer->addSoundMonitor(drawSoundWidget); //Why QThread isn't QObject ?!?
     soundPlayer->addSoundMonitor(spectrumView);        //Why QThread isn't QObject ?!?
     soundPlayer->addSoundMonitor(scopeView);        //Why QThread isn't QObject ?!?
+    midiInput->setSoundPlayer(soundPlayer);
 
 
     centralLayout->hide();
@@ -457,14 +483,14 @@ Freecycle::Freecycle()
     menuBar()->setItemEnabled(4,FALSE);
 
     QPopupMenu* helpPopupMenu=new QPopupMenu( this );
+    helpPopupMenu->insertItem( tr("Help"),this,SLOT(help()));
     helpPopupMenu->insertItem( tr("About"),this,SLOT(about()));
     menuBar()->insertItem( tr("&Help"), helpPopupMenu,5);
 
 
     openFileDialog=new MyOpenFileDialog(soundManager,soundPlayer,fileManager);
     saveFileDialog=new MySaveFileDialog();
-    projectManager=new ProjectManager(soundManager,
-                                                                             envelopeControllerWidget->getEnvData());
+    projectManager=new ProjectManager(soundManager,envelopeControllerWidget->getEnvData());
     projectManager->setLoopMarkers(drawSoundWidget->getLeftLoopMarker(),
                                                                     drawSoundWidget->getRightLoopMarker());
 
@@ -472,8 +498,12 @@ Freecycle::Freecycle()
     connect(projectManager,SIGNAL(envelopeDataChanged()),
                 envelopeControllerWidget, SLOT(envDataChanged()));
     connect(projectManager,SIGNAL(addLadspa(int)),this,SLOT(addEffectFrame(int)));
-    connect(projectManager,SIGNAL(addLockedBeatlines(QValueList<long>)),
-            drawSoundWidget,SLOT(setLockedBeatLines(QValueList <long>)));
+    connect(projectManager,SIGNAL(addLockedBeatlines(QValueList<beatline_data>)),
+            drawSoundWidget,SLOT(initLockedBeatLines(QValueList <beatline_data>)));
+    connect(projectManager,SIGNAL(addStackElementS(long,long,QString,long,int,bool )),
+            drawSoundWidget,SLOT(addStackElement(long,long,QString,long,int,bool )));
+    connect(projectManager,SIGNAL(bpmModifierChangeS(QString )),
+            bpmModifDropDown,SLOT(setStringModifier(QString)));
 
     saveProjectDialog=new MySaveProjectDialog(projectManager);
     loadProjectDialog=new MyLoadProjectDialog(projectManager);
@@ -494,16 +524,23 @@ Freecycle::Freecycle()
 
     //midiOutputThread=new MidiOutputThread();
     //midiOutputThread->start();
+    
+    
+    
+    midiInputThread=new MidiInputThread(midiInput);
+    midiInputThread->start();
 
-    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< long > )),
-        soundManager,SLOT(beatLinesChanged(QValueList< long > )));
-    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< long > )),
-        soundManager,SLOT(lockedBeatLinesChanged(QValueList< long > )));
+    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< beatline_data > )),
+        soundManager,SLOT(beatLinesChanged(QValueList< beatline_data > )));
+    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< beatline_data > )),
+        soundManager,SLOT(lockedBeatLinesChanged(QValueList< beatline_data > )));
 
     filePopupMenu->setItemEnabled(idsavedialog,FALSE);
     filePopupMenu->setItemEnabled(idsaveasdialog,FALSE);
     filePopupMenu->setItemEnabled(idsavechunksdialog,FALSE);
+    filePopupMenu->setItemEnabled(idsaveprojectasdialog,FALSE);
     filePopupMenu->setItemEnabled(idsaveprojectdialog,FALSE);
+    filePopupMenu->setItemEnabled(idsaveloopmidifile,FALSE);
 }
 
 void Freecycle::changeZoom(int value){
@@ -616,7 +653,9 @@ void Freecycle::load( const QString &fileName )
         filePopupMenu->setItemEnabled(idsavedialog,TRUE);
         filePopupMenu->setItemEnabled(idsaveasdialog,TRUE);
         filePopupMenu->setItemEnabled(idsavechunksdialog,TRUE);
+        filePopupMenu->setItemEnabled(idsaveprojectasdialog,TRUE);
         filePopupMenu->setItemEnabled(idsaveprojectdialog,TRUE);
+        filePopupMenu->setItemEnabled(idsaveloopmidifile,TRUE);
 
         if(effectFrame) effectFrame->reinit();
         if(!topBox->isVisible()) topBox->show();
@@ -637,7 +676,6 @@ void Freecycle::playFromTo(long x1,long x2){
     soundPlayer->setOneTimeLoop(x1,x2);
     soundPlayer->setPlayingOnce(TRUE);
     play();
-    soundPlayer->setPosition(x1);
 }
 
 void Freecycle::play(){
@@ -701,79 +739,65 @@ void Freecycle::saveAs()
 
 void Freecycle::saveChunks()
 {
-    int size=drawSoundWidget->getNbBeats();
+    int size=drawSoundWidget->getNbLockedBeats();
     if(size==0){
-        QMessageBox::critical(this,"Problem!","No beatlines, no chunks! (:P)",0);
+        QMessageBox::critical(this,"Problem!","No locked beatlines, no chunks! (:P)",0);
     }else{
         QString fn=NULL;
         saveFileDialog->setBps(soundManager->getBps());
         if ( saveFileDialog->exec() == QDialog::Accepted )
             fn=saveFileDialog->selectedFile();
         if ( !fn.isEmpty() ) {
-            QMemArray <long>beatlines(0);
-            long* temp=new long[size];
-            drawSoundWidget->getBeats(temp);
-            std::sort(&temp[0],&temp[size-1]);
+            long start=0;
+            long end=soundManager->getFrames();
+            QValueList <beatline_data> beatlinesVL;
             if(saveFileDialog->exportMode()==MySaveFileDialog::EXPORT_LOOP){
                 if(drawSoundWidget->hasLoopSet()){
                     long left=drawSoundWidget->getLeftLoopMarkerSample();
                     long right=drawSoundWidget->getRightLoopMarkerSample();
-                    for (int i=0;i<size;i++){
-                        if(temp[i]>=left && temp[i]<=right){
-			/*
-                            if(beatlines.size()==0 && temp[i]!=left){ //include left marker if not present
-                                beatlines.resize(beatlines.size()+1);
-                                beatlines[beatlines.size()-1]=left;
-				cout<<"included: "<<left<<endl;
-                            }
-			    */
-                            beatlines.resize(beatlines.size()+1);
-                            beatlines[beatlines.size()-1]=temp[i];
-			    cout<<"included: "<<temp[i]<<endl;
-                        }
-                    }
-		    /*
-                    if(beatlines[beatlines.size()-1]!=right){ //include right marker if not present
-                        beatlines.resize(beatlines.size()+1);
-                        beatlines[beatlines.size()-1]=right;
-			cout<<"included: "<<right<<endl;
-                    }
-		    */
+                    beatlinesVL=drawSoundWidget->selectLockedBeatlinesList(left,right);
+                    start=left;
+                    end=right;
                 }else{
                     QMessageBox::critical(this,"Problem!","Invalid loop!",0);
                     return;
                 }
             }else{
-                beatlines.assign(temp,size);
-                std::sort(&beatlines[0],&beatlines[size-1]);
-                if(beatlines[0]!=0){ //include first frame if not present
-                    beatlines.resize(beatlines.size()+1);
-                    beatlines[beatlines.size()-1]=0;
-                    std::sort(&beatlines[0],&beatlines[size-1]);
-                }
-                if(beatlines[size-1]!=soundManager->getFrames()-1){ //include last frame if not present
-                    beatlines.resize(beatlines.size()+1);
-                    beatlines[beatlines.size()-1]=soundManager->getFrames()-1;
-                    std::sort(&beatlines[0],&beatlines[size-1]);
-                }
+                beatlinesVL=drawSoundWidget->getLockedBeatList();
             }
             cout<<"Export bits per sample: "<<saveFileDialog->getExportBitsPerSample()<<" code: "<<soundManager->toBpsCode(saveFileDialog->getExportBitsPerSample())<<"\n";
             /*
             if(saveFileDialog->getAttackTime()!=0){
                 Envelope* env=new Envelope();
-                env->setEnvDurationAndSamplerate(saveFileDialog->getAttackTime(),soundManager->getRate());
+                env->setEnvDurationAndSamplerate(saveFileDialog->getAttackTime(),
+                                                 soundManager->getRate());
                 soundManager->addPostProcessEffect(env);
             }
             */
+            
             QStringList* filenames=
-                soundManager->saveChunksTo(fn,beatlines.data(),beatlines.size(),
-                                     soundManager->toBpsCode(saveFileDialog->getExportBitsPerSample()));
+                soundManager->saveChunksTo(fn,beatlinesVL,
+                              soundManager->toBpsCode(saveFileDialog->getExportBitsPerSample()));
+            if(saveFileDialog->assignMissingMIDINotes()){
+                beatlinesVL=drawSoundWidget->assignMissingMidi(beatlinesVL);
+            }
             if(saveFileDialog->exportMidifile()){
-                midiManager->writeMidiFile(fn,beatlines.data(),
-                                                                    beatlines.size(),drawSoundWidget->getUsedBPM());
+                midiManager->writeMidiFile(fn,beatlinesVL,drawSoundWidget->getUsedBPM());
             }
             if(saveFileDialog->exportAKP()){
-                akpManager->writeAKPFile(fn,filenames);
+                akpManager->writeAKPFile(fn,filenames,beatlinesVL);
+            }
+            if(saveFileDialog->exportSF2()){
+                sf2Manager->writeSF2File(fn,filenames,beatlinesVL);
+            }
+            if(saveFileDialog->exportDLS()){
+                dlsManager->writeDLS2File(fn,filenames,beatlinesVL);
+            }
+            if(saveFileDialog->exportGiga()){
+                gigaManager->writeGigaFile(fn,filenames,beatlinesVL);
+            }
+            if(saveFileDialog->removeChunks()){
+                fileManager->deleteDir(fn);
             }
             statusBar()->message( tr( "File %1 saved to chunks" ).arg( fn), 2000 );
             delete filenames;
@@ -806,7 +830,7 @@ void Freecycle::about()
                         "\n\nCredits:\n\n"
                         "JACK layer  and LADSPA control port init taken from the AlsaModularSynth project by Matthias Nagorni"
                             "\nhttp://alsamodular.sourceforge.net/\n\n"
-                        "/dev/dsp layer borrowed from the libsndfile examples by Erik de Castro Lopo\n"
+                        "OSS layer borrowed from the libsndfile examples by Erik de Castro Lopo\n"
                             "http://www.mega-nerd.com/libsndfile/\n\n"
                         "Midi file generation code stolen from the Waon project\n"
                             "Copyright (C) 1998 Kengo ICHIKI (ichiki@geocities.com)\n\n"
@@ -913,7 +937,8 @@ void Freecycle::removeEffect(){
         soundManager->applyEnvelope();
     else soundManager->zapInstantEBuffer();
     if(effectFrame){
-        effectFrame->close();
+        effectFrame->close(TRUE);
+        effectFrame=0;
     }
     drawSoundWidget->zapWaveCanvasBuffers();
     drawAllSoundWidget->zapWaveCanvasBuffers();
@@ -942,11 +967,13 @@ void Freecycle::addEffectFrame(int uniqueId){
                     effectFrame,SLOT(setContentsPos(int,int)));
     connect(soundManager,SIGNAL(reinitLADSPAValues()),effectFrame,SLOT(reinitLADSPAValues()));
     /*
-    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< long > )),
-            effectFrame,SIGNAL(beatLinesChanged(QValueList< long > )));
+    connect(drawSoundWidget,SIGNAL(beatLinesChanged(QValueList< beatline_data > )),
+            effectFrame,SIGNAL(beatLinesChanged(QValueList< beatline_data > )));
     */
-    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< long > )),
-            effectFrame,SLOT(lockedBeatLinesChanged(QValueList< long > )));
+    connect(drawSoundWidget,SIGNAL(lockedBeatLinesChanged(QValueList< beatline_data > )),
+            effectFrame,SLOT(lockedBeatLinesChanged(QValueList< beatline_data > )));
+
+    connect(soundManager,SIGNAL(cropped(long, long )),effectFrame,SIGNAL(cropS(long, long )));
     effectFrame->lockedBeatLinesChanged(drawSoundWidget->getLockedBeatList());
 
     effectFrame->show();
@@ -1193,16 +1220,17 @@ void Freecycle::numPad1Pressed()
 
 
 /*!
-    \fn Freecycle::saveProject()
+    \fn Freecycle::saveProjectAs()
  */
-void Freecycle::saveProject()
+void Freecycle::saveProjectAs()
 {
     QString fn=NULL;
     if ( saveProjectDialog->exec() == QDialog::Accepted ){
         fn=saveProjectDialog->selectedFile();
         projectManager->setFilename(fn);
-        projectManager->setLockedBeatLines(
-                drawSoundWidget->getLockedBeatList());
+        projectManager->setLockedBeatLines(drawSoundWidget->getLockedBeatList());
+        projectManager->setLockedBeatRects(drawSoundWidget->getLockedBeatRects());
+        projectManager->setBpmModifier(bpmModifDropDown->getStringModifier());
         projectManager->saveProject();
     }
 }
@@ -1217,7 +1245,82 @@ void Freecycle::loadProject()
     if ( loadProjectDialog->exec() == QDialog::Accepted ){
             fn=loadProjectDialog->selectedFile();
             projectManager->loadProject(fn);
+            setCaption(release+QString(" - ")+fn);
             drawSoundWidget->analysisFinished(); //force bpm computation based on loaded beatlines
+            drawSoundWidget->getCanvas()->update();
     }
 
 }
+
+
+/*!
+    \fn Freecycle::configureMIDI()
+ */
+void Freecycle::configureMIDI()
+{
+    midiConfigDialog->exec();
+}
+
+
+/*!
+    \fn Freecycle::saveLoopMidiFile()
+ */
+void Freecycle::saveLoopMidiFile()
+{
+    QString filename="export.mid";
+
+    QString fn = QFileDialog::getSaveFileName( filename, "MIDI files ( *.mid )",
+					       this );
+
+    if ( !fn.isEmpty() ) {
+        QValueList <beatline_data> beatlinesVL;
+        if(drawSoundWidget->hasLoopSet()){
+            long left=drawSoundWidget->getLeftLoopMarkerSample();
+            long right=drawSoundWidget->getRightLoopMarkerSample();
+            beatlinesVL=drawSoundWidget->selectLockedBeatlinesList(left,right);
+        }else{
+            QMessageBox::critical(this,"Problem!","Invalid loop!",0);
+            return;
+        }
+    
+        midiManager->writeMidiFile(fn,beatlinesVL,drawSoundWidget->getUsedBPM());
+        statusBar()->message( tr( "Midi File  %1 saved" ).arg( fn), 2000 );
+    } else {
+	statusBar()->message( tr("Saving aborted"), 2000 );
+    }
+}
+
+
+/*!
+    \fn Freecycle::saveProject()
+ */
+void Freecycle::saveProject()
+{
+    if(projectManager->getFilename()!=""){
+        projectManager->setLockedBeatLines(drawSoundWidget->getLockedBeatList());
+        projectManager->setLockedBeatRects(drawSoundWidget->getLockedBeatRects());
+        projectManager->setBpmModifier(bpmModifDropDown->getStringModifier());
+        projectManager->saveProject();
+    }else saveProjectAs();
+}
+
+
+/*!
+    \fn Freecycle::loadProject(QString fn)
+ */
+void Freecycle::loadProject(QString fn)
+{
+    projectManager->loadProject(fn);
+    setCaption(release+QString(" - ")+fn);
+    drawSoundWidget->analysisFinished(); //force bpm computation based on loaded beatlines
+    drawSoundWidget->getCanvas()->update();
+}
+
+
+/*!
+    \fn Freecycle::help()
+ */
+void Freecycle::help()
+{
+    helpManager->open();
+}
