@@ -420,7 +420,7 @@ void SoundHolder::saveChunkToAs(QString filename,long start,long end,
 int SoundHolder::loadBuffer(QString path,bufferType type){
     SNDFILE* tmp_sndfile= sf_open(path, SFM_READ, &sfinfo);
     if (!tmp_sndfile) return 1;
-    sf_command(tmp_sndfile,SFC_CALC_SIGNAL_MAX,&maxValue,sizeof(double));
+    sf_command(tmp_sndfile,SFC_CALC_NORM_SIGNAL_MAX,&maxValue,sizeof(double));
 
 
     //std::cout<<maxValue;
@@ -562,30 +562,68 @@ bool SoundHolder::getHasLADSPA()
 /*!
     \fn SoundHolder::insertFileAt(QString filename,long start_sample)
  */
-void SoundHolder::insertFileAt(QString filename,long start_sample)
+long SoundHolder::insertFileAt(QString filename,long start_sample)
 {
-    SNDFILE* tmp_sndfile= sf_open(filename, SFM_READ, &sfinfo);
-    if (!tmp_sndfile) return ;
+    SF_INFO tempsfinfo;
+    SNDFILE* tmp_sndfile= sf_open(filename, SFM_READ, &tempsfinfo);
+    if (!tmp_sndfile) return 0;
 
-    float* wavefile=new float[sfinfo.frames*2/*sfinfo.channels*/];
-    sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)sfinfo.frames);
+    float* wavefile=new float[tempsfinfo.frames*2/*sfinfo.channels*/];
+    sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)tempsfinfo.frames);
 
-    enlargeBuffersAt(start_sample,sfinfo.frames);
+    enlargeBuffersAt(start_sample,tempsfinfo.frames);
 
     //demux channels
     int count=0;
     for (int i=start_sample;i<start_sample+sfinfo.frames;i++){
-            //getLeftChannel(type)[i]=wavefile[count];
+            getLeftChannel(ORIGINAL_WAVE)[i]=wavefile[count];
         if(channels==2){
-            //getRightChannel(type)[i]=wavefile[count+1];
-        }else if(channels==1){
-            //getRightChannel(type)[i]=wavefile[count];
+            getRightChannel(ORIGINAL_WAVE)[i]=wavefile[count+1];
         }
     }
     //end demux channels
 
-    zaparr(wavefile);
     sf_close(tmp_sndfile);
+    setNbFrames(getFrames()+tempsfinfo.frames);
+    return tempsfinfo.frames;
+}
+
+
+/*!
+    \fn SoundHolder::replaceFileAt(QString filename,long start_sample,long end_sample)
+ */
+void SoundHolder::replaceFileAt(QString filename,long start_sample,long end_sample)
+{
+    SF_INFO tempsfinfo;
+    SNDFILE* tmp_sndfile= sf_open(filename, SFM_READ, &tempsfinfo);
+    if (!tmp_sndfile){
+        cout<<"Error opening "<<filename<<endl;
+        return;
+    }
+
+    float* wavefile=new float[tempsfinfo.frames*2/*sfinfo.channels*/];
+    sf_readf_float(tmp_sndfile, wavefile,(sf_count_t)tempsfinfo.frames);
+
+    //demux channels
+    int count=0;
+    for (int i=start_sample;i<end_sample;i++){
+        if(i<start_sample+tempsfinfo.frames){
+            getLeftChannel(ORIGINAL_WAVE)[i]=wavefile[count++];
+            if(channels==2){
+                getRightChannel(ORIGINAL_WAVE)[i]=wavefile[count++];
+            }
+        }else{
+            getLeftChannel(ORIGINAL_WAVE)[i]=0;
+            if(channels==2){
+                getRightChannel(ORIGINAL_WAVE)[i]=0;
+            }
+        }
+    }
+    //end demux channels
+
+    sf_close(tmp_sndfile);
+    zaparr(wavefile);
+    cout<<"File inserted from "<<start_sample<<" to "<<end_sample<<"\n";
 }
 
 
