@@ -71,10 +71,9 @@ QMemArray<long> SoundManager::analyse(const double treshold,
     #ifdef HAS_AUBIO
         else if(analysisMethod==COMPLEX_DOMAIN || analysisMethod==HFC
                             || analysisMethod==PHASE_DIFF || analysisMethod==COMPLEX_DOMAIN)
-        return analyse_aubio_onsets(treshold,window,start_sample,end_sample);
-        //else
-        //    return error;
+        return analyse_aubio_onsets(treshold,window,start_sample,end_sample);        
     #endif // HAS_AUBIO
+    //else return error;
 }
 
 QMemArray<long> SoundManager::analyse_freq_domain(const double treshold,
@@ -672,6 +671,7 @@ void SoundManager::resample(list<long>sortedPos,double new_to_oldBPM_ratio)
 #endif;
         QProgressDialog progress("Time stretching/expanding...","Abort",getFrames(),NULL,"progress",TRUE);
         progress.setMinimumDuration(2000);
+        progress.setProgress(0);
         SoundTouch* st=new SoundTouch();
 	st->setSampleRate(getRate());
 	st->setChannels(getChannels());
@@ -725,10 +725,9 @@ void SoundManager::resample(list<long>sortedPos,double new_to_oldBPM_ratio)
                                                                     tmp_wave2,currentPos,returned);
 #endif
                        currentPos+=returned;
-                       if(true/*i%1000==0*/){
+                       if(true){
                             progress.setProgress(currentPos);
-                            progress.setLabelText("Analyzing input signal...(frame: "+QString::number(currentPos)+")");
-                            QApplication::flushX();
+                            progress.setLabelText("Time stretching/expanding...(frame: "+QString::number(currentPos)+")");
                         }
 		}while(returned!=0);
                 zaparr(tmp_wave);
@@ -764,11 +763,14 @@ void SoundManager::resample(list<long>sortedPos,double new_to_oldBPM_ratio)
         if(getHasLADSPA()) initBuffer(AFTER_LADSPA,getFrames());
         if(getHasEnvelope()) initBuffer(AFTER_ENVELOPE,getFrames());
         getEffects()->setNbFrames(getFrames());
+        cout<<"timestretch end\n";
         emit(reinitLADSPAValues());
-        if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); //reapply current plugin
-        applyEnvelope();
         zapInstantEBuffer();
-        emit(waveChanged());
+        if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); //reapply current plugin
+        else if(getHasEnvelope()) applyEnvelope();
+        else emit(waveChanged());
+
+
 }
 
 
@@ -922,27 +924,27 @@ int SoundManager::loadBuffer(QString path,long start_sample,long end_sample){
 
 
 /*!
-    \fn SoundManager::saveChunksTo(QString,long*)
+    \fn SoundManager::saveChunksTo(QString filename,QValueList <beatline_data> bld,int format)
  */
-QStringList* SoundManager::saveChunksTo(QString filename,long beats[],int length,int format)
+QStringList* SoundManager::saveChunksTo(QString filename,QValueList <beatline_data> bld,int format)
 {
     QDir dir;
     dir.mkdir(filename);
     QStringList* resultFilenames=new QStringList();
-    std::sort(beats,beats+length);
-    std::cout<<"size: "<<length<<"\n";
-    for (int i=0;i<length-1;i++){
-        QString tempfilename=QString::number(i).rightJustify(8,'0').append(".wav").prepend(filename+"/");
-        saveChunkToAs(tempfilename,beats[i],beats[i+1],format,0);
+    qHeapSort(bld);
+    for (int i=0;i<bld.count()-1;i++){
+        QString tempfilename=
+            QString::number(i).rightJustify(8,'0').append(".wav").prepend(filename+"/");
+        saveChunkToAs(tempfilename,bld[i].position,bld[i+1].position,format,0);
         resultFilenames->push_back(tempfilename.section( '/', -1 ));
     }
     postProcessEffects->clear();
     return resultFilenames;
 }
 
-QStringList* SoundManager::saveChunksTo(QString filename,long beats[],int length)
+QStringList* SoundManager::saveChunksTo(QString filename,QValueList <beatline_data> bld)
 {
-    return saveChunksTo(filename,beats,length,sfinfo.format);
+    return saveChunksTo(filename,bld,sfinfo.format);
 }
 
 
@@ -968,7 +970,6 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
     //end audio ports
 
     effect->init(getRate(),inputBufferLeft,inputBufferRight,outputBufferLeft,outputBufferRight);
-    //effect->startControlPortValues(start_sample);
     effect->startPortValues(start_sample);
     cout<<"applying plugin from sample: "<<start_sample<<" to sample:"<<end_sample<<"\n";
     long processedSamples=0;
@@ -996,7 +997,9 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
     if(hasEnvelope){
         applyEnvelope();
     }else{
-        this->zapInstantEBuffer();
+        //force spectrogram repaint
+        if(hiresfreq_subband_energies) zaparr(hiresfreq_subband_energies);
+        //this->zapInstantEBuffer();
         emit(waveChanged()); //is ..
     }
 }
@@ -1004,7 +1007,6 @@ void SoundManager::applyPlugin(LADSPAEffect* effect,long start_sample, long end_
 
 void SoundManager::applyPlugin(LADSPAEffect* effect){
     applyPlugin(effect,0,getFrames());
-    emit(waveChanged());
 }
 
 
@@ -1333,8 +1335,8 @@ int w, bool ssm)
         substractSpectralMean=ssm;
 
         //recompute_subband_energies();
-        //this->zapInstantEBuffer();
-        if(!getHasEnvelope() && !getHasLADSPA()) this->zapInstantEBuffer();
+        this->zapInstantEBuffer();
+        //if(!getHasEnvelope() && !getHasLADSPA()) this->zapInstantEBuffer();
         emit(fftParamsChanged());
         emit(analysisParamsChanged());
 }
@@ -1711,17 +1713,21 @@ void SoundManager::applyEnvelope()
             long r_duration=(envelopeData->release*envelopeData->env_duration/1000.0)*float(getRate());
             long s_duration=applyNbFrames-r_duration-ahd_duration;
             for (long j=0;j<lockedBeatLines.count();j++){
-                long end=j<lockedBeatLines.count()-1?lockedBeatLines[j+1]:getFrames();
-                for (long i=lockedBeatLines[j];i<end;i++){
+                long end=j<lockedBeatLines.count()-1?lockedBeatLines[j+1].position:getFrames();
+                for (long i=lockedBeatLines[j].position;i<end;i++){
                     if(envelopeData->env_mode!=EnvelopeData::FIT_TO_SAMPLES &&
-                    i-lockedBeatLines[j]<applyNbFrames){
-                        outputBufferLeft[i]=inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j]];
-                        outputBufferRight[i]=inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j]];
+                    i-lockedBeatLines[j].position<applyNbFrames){
+                        outputBufferLeft[i]=
+                            inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j].position];
+                        outputBufferRight[i]=
+                            inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j].position];
                     }else if(envelopeData->env_mode==EnvelopeData::FIT_TO_SAMPLES){
-                        if(i-lockedBeatLines[j]<=ahd_duration){
-                            outputBufferLeft[i]=inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j]];
-                            outputBufferRight[i]=inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j]];
-                        }else if(i-lockedBeatLines[j]>ahd_duration &&i<end-r_duration){
+                        if(i-lockedBeatLines[j].position<=ahd_duration){
+                            outputBufferLeft[i]=
+                                inputBufferLeft[i]*envelopeArray[i-lockedBeatLines[j].position];
+                            outputBufferRight[i]=
+                                inputBufferRight[i]*envelopeArray[i-lockedBeatLines[j].position];
+                        }else if(i-lockedBeatLines[j].position>ahd_duration &&i<end-r_duration){
                             //apply sustain
                             outputBufferLeft[i]=inputBufferLeft[i]*envelopeData->sustain;
                             outputBufferRight[i]=inputBufferRight[i]*envelopeData->sustain;
@@ -1746,16 +1752,18 @@ void SoundManager::applyEnvelope()
         }
     }
     cout<<"Envelope applied\n";
-    this->zapInstantEBuffer();
+    //force spectrogram repaint
+    if(hiresfreq_subband_energies) zaparr(hiresfreq_subband_energies);
+    //this->zapInstantEBuffer();
     //emit(waveChanged(start_sample,end_sample));  //should be..
     emit(waveChanged()); //is ..
 }
 
 
 /*!
-    \fn SoundManager::lockedBeatLinesChanged(QValueList<long> beatlines)
+    \fn SoundManager::lockedBeatLinesChanged(QValueList<beatline_data> beatlines)
  */
-void SoundManager::lockedBeatLinesChanged(QValueList<long> beatlines)
+void SoundManager::lockedBeatLinesChanged(QValueList<beatline_data> beatlines)
 {
     lockedBeatLines=beatlines;
     applyEnvelope();
@@ -1763,9 +1771,9 @@ void SoundManager::lockedBeatLinesChanged(QValueList<long> beatlines)
 
 
 /*!
-    \fn SoundManager::beatLinesChanged(QValueList<long> beatlines)
+    \fn SoundManager::beatLinesChanged(QValueList<beatline_data> beatlines)
  */
-void SoundManager::beatLinesChanged(QValueList<long> beatlines)
+void SoundManager::beatLinesChanged(QValueList<beatline_data> beatlines)
 {
     beatLines=beatlines;
 }
@@ -1835,7 +1843,37 @@ bool SoundManager::canOpen(QString file)
     bool openable=FALSE;
     SF_INFO tempsfinfo;
     SNDFILE* tmp_sndfile= sf_open(file, SFM_READ, &tempsfinfo);
-    if(tmp_sndfile==0) openable=FALSE; else openable=TRUE;
+    if(tmp_sndfile) openable=TRUE; else openable=FALSE;
     sf_close(tmp_sndfile);
     return openable;
 }
+
+
+/*!
+    \fn SoundManager::insertFileAt(QString filename,long start_sample)
+ */
+long SoundManager::insertFileAt(QString filename,long start_sample)
+{
+    long amount=SoundHolder::insertFileAt(filename,start_sample);
+    zapInstantEBuffer();
+    if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); 
+    else if(getHasEnvelope()) applyEnvelope();
+    else emit(waveChanged());
+    emit(inserted(start_sample, amount)); //not used
+    return amount;
+}
+
+
+
+/*!
+    \fn SoundManager::replaceFileAt(QString filename, long start, long end)
+ */
+void SoundManager::replaceFileAt(QString filename, long start, long end)
+{
+    SoundHolder::replaceFileAt(filename,start,end);
+    zapInstantEBuffer();
+    if(getEffects()->getCurrent()) applyPlugin(getEffects()->getCurrent()); 
+    else if(getHasEnvelope()) applyEnvelope();
+    else emit(waveChanged());
+    emit(replaced(start, end)); //not used
+}
