@@ -21,7 +21,6 @@
 #define SOUNDMANAGER_H
 #include "defs.h"
 
-//#define INTEGER_SAMPLES
 #include <soundtouch/SoundTouch.h>
 #include <qmemarray.h>
 #include <qstring.h>
@@ -33,6 +32,7 @@
 #include <qprogressdialog.h>
 #include <qapplication.h>
 #include <qptrlist.h>
+#include <qvaluelist.h>
 #include <qeventloop.h>
 #include <ladspa.h>
 #include <dlfcn.h>
@@ -51,6 +51,8 @@
 #include "soundholder.h"
 #include "fftholder.h"
 #include "envelopedata.h"
+#include "beatlinedata.h"
+
 
 /**
 @author Predrag Viceic
@@ -98,6 +100,7 @@ public:
     bool hasSound();
     double getMaxValue();
     void crop(long,long);
+    long insertFileAt(QString filename,long start_sample);
     void setFileManager(FileManager*);
     void pushSoundBuffer(bufferType type);
     void saveSoundBuffer(long,long);
@@ -105,16 +108,14 @@ public:
     bool previousSoundBuffer(bufferType type);
     bool previousSoundBuffer(long,long);
     bool nextSoundBuffer();
-    QStringList* saveChunksTo(QString,long[],int,int format);
-    QStringList* saveChunksTo(QString,long[],int);
+    QStringList* saveChunksTo(QString filename,QValueList <beatline_data> bld,int format);
+    QStringList* saveChunksTo(QString filename,QValueList <beatline_data> bld);
     void saveChunkTo(QString,long start_sample,long end_sample,long start_at=0);
 
 
 
 
     LADSPAEffects* getEffects();
-    short* insertShortArray(short*,short*,long,long);
-    float* insertArray(float*,float*,long,long);
     void zapInstantEBuffer();
     float** compute_subband_energies(int,int,int,int,long,long,int,int,float&,float&,bufferType type);
     void setAnalysisMethod(int);
@@ -155,8 +156,7 @@ public:
     QString getTempDir();
     bool canOpen(QString file);
 
-    QValueList<long> beatLines;
-    QValueList<long> lockedBeatLines;
+    
 
 protected:
     FileManager* fileManager;
@@ -174,15 +174,20 @@ protected:
     bool playingBandOnly;
     bool substractSpectralMean;
     QMemArray <beat> foundBeats;
+    QValueList <beatline_data> beatlines;
+    QValueList <beatline_data> lockedBeatlines;
     EnvelopeData* envelopeData;
     float* envelopeArray;
 
 private:
-    int loadBuffer(QString,long,long);
+    
     float* instant_buffer;
     int startBand;
     int endBand;
     bool inversionBand;
+    QValueList <beatline_data> beatLines;
+    QValueList <beatline_data> lockedBeatLines;
+    int loadBuffer(QString,long,long);
 
 public slots:
     void applyPlugin(LADSPAEffect*);
@@ -191,9 +196,10 @@ public slots:
     void startBandChanged(int);
     void endBandChanged(int);
     void applyEnvelope();
-    void lockedBeatLinesChanged(QValueList<long> beatlines);
-    void beatLinesChanged(QValueList<long> beatlines);
+    void lockedBeatLinesChanged(QValueList<beatline_data> beatlines);
+    void beatLinesChanged(QValueList<beatline_data> beatlines);
     void envelopeChanged(EnvelopeData* envData);
+    void replaceFileAt(QString filename, long start, long end);
 
 signals:
     void waveChanged();
@@ -204,6 +210,8 @@ signals:
     void fftParamsChanged();
     void analysisParamsChanged();
     void reinitLADSPAValues();
+    void replaced(long start, long end);
+    void inserted(long,long);
 };
 
 #endif
