@@ -78,10 +78,11 @@ SoundPlayer::SoundPlayer()
         rightLoop=0;
         output=OUTPUT_NONE;
         playFromFileV=FALSE;
-        previewFileJack=0;
+        previewFile=0;
         savedLeftLoop=-1;
         savedRightLoop=-1;
         closeV=FALSE;
+        portaudio_stream=0;
 }
 
 
@@ -90,37 +91,29 @@ SoundPlayer::~SoundPlayer()
     zaparr(bufferLR);
     if(output==OUTPUT_DEVDSP) ::close (devdsp_audio_device) ;
     else if(output==OUTPUT_JACK) closeJack();
+    else if(output==OUTPUT_PORTAUDIO) closePortaudio();
 }
 
 void SoundPlayer::playDevDsp(){
-        if(output==OUTPUT_DEVDSP){
-            while(position<rightLoop && playing){
-                    if(position>=0 && position<soundManager->getFrames()){
-                        if(soundManager->isPlayingBandOnly()){
-                                soundManager->exportMultiplexedShortChannelBand(bufferLR,position,position+buffer_len);
-                        	position+=buffer_len;
-			}else{
-				if(position+buffer_len<rightLoop){
-                                    soundManager->exportMultiplexedShortBuffer(bufferLR, position,position+buffer_len);
-                                       if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, buffer_len*2);
-					write (devdsp_audio_device,bufferLR,buffer_len*2* sizeof (short)) ;
-					position+=buffer_len;
-                                }else if (position<rightLoop){
-                                    soundManager->exportMultiplexedShortBuffer(bufferLR, position,
-                                            rightLoop);
-                                        if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, (rightLoop-position)*2);
-					write (devdsp_audio_device,bufferLR,(rightLoop-position)*2* sizeof (short));
-					position=rightLoop;
-                                }else position=rightLoop;
-			}
-                        positionChanged(position);
-                    }
-            }
-            if(playingOnce){
-                this->stopPlay();
-                playingOnce=FALSE;
-                resetLoop();
-            }
+        while(position<rightLoop && playing){
+            if(position+buffer_len<rightLoop){
+                soundManager->exportMultiplexedShortBuffer(bufferLR, position,position+buffer_len);
+                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, buffer_len*2);
+                    write (devdsp_audio_device,bufferLR,buffer_len*2* sizeof (short)) ;
+                    position+=buffer_len;
+            }else if (position<rightLoop){
+                soundManager->exportMultiplexedShortBuffer(bufferLR, position,
+                        rightLoop);
+                    if (NOT_NATIVE_ENDIAN) swap_endian(bufferLR, (rightLoop-position)*2);
+                    write (devdsp_audio_device,bufferLR,(rightLoop-position)*2* sizeof (short));
+                    position=rightLoop;
+            }else position=rightLoop;
+            positionChanged(position);
+        }
+        if(playingOnce){
+            this->stopPlay();
+            playingOnce=FALSE;
+            resetLoop();
         }
 }
 
@@ -129,6 +122,7 @@ void SoundPlayer::stopPlay(){
         if(playing){
             playing=false;
             positionChanged(leftLoop);
+            if(output==OUTPUT_PORTAUDIO) stopPAStream();
         }
 }
 
@@ -253,6 +247,8 @@ bool SoundPlayer::isPlaying()
 void SoundPlayer::setPlaying(bool _pl)
 {
     playing=_pl;
+    if(output==OUTPUT_PORTAUDIO && playing) startPAStream();
+    else if(output==OUTPUT_PORTAUDIO && !playing) stopPAStream();
 }
 
 
@@ -263,7 +259,8 @@ void SoundPlayer::setLoop(long l,long r)
 {
     leftLoop=l<0?0:l;
     rightLoop=r>=soundManager->getFrames()?soundManager->getFrames()-1:r;
-    if(!playing) position=leftLoop;
+    position=leftLoop;
+    //cout<<"position set in setLoop("<<l<<","<<r<<") at "<<position<<endl;
 }
 
 /*!
@@ -275,7 +272,8 @@ void SoundPlayer::setOneTimeLoop(long l,long r)
     savedRightLoop=rightLoop;
     leftLoop=l;
     rightLoop=r;
-    if(!playing) position=leftLoop;
+    position=leftLoop;
+    //cout<<"position set in setOneTimeLoop("<<l<<","<<r<<") at "<<position<<endl;
 }
 
 
@@ -304,7 +302,7 @@ int SoundPlayer::connect_jack()
 
 
     std::cout<<"Querying Jack...\n";
-    QString jackName="Freecycle";
+    QString jackName="Freecycle "+QString::number(QDateTime::currentDateTime().toTime_t());
     if ((jack_handle = jack_client_new (jackName.latin1())) == 0){
         fprintf (stderr, "Can't connect to JACK\n");
         return 1;
@@ -316,8 +314,8 @@ int SoundPlayer::connect_jack()
     //periodsize = MAXIMUM_PERIODSIZE;
 
 
-        jack_out [0] = jack_port_register (jack_handle, "out L", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
-        jack_out [1] = jack_port_register (jack_handle, "out R", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+    jack_out [0] = jack_port_register (jack_handle, "out L", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+    jack_out [1] = jack_port_register (jack_handle, "out R", JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 
 
     if (jack_activate (jack_handle))
@@ -366,17 +364,22 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
                  (jack_default_audio_sample_t *)(jack_port_get_buffer (jack_out [0], nframes));
     jack_default_audio_sample_t * p2 =
                  (jack_default_audio_sample_t *)(jack_port_get_buffer (jack_out [1], nframes));
-    if(!soundManager) return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
+    if(!(p1 && p2)){
+        cout<<"Jack buffers unavailable!!\n";
+        resetLoop();
+        playing=FALSE;
+        playingOnce=FALSE;
+        return 0;
+    }
     if(nframes>buffer_len) setJackBuffersLen(nframes);
-
     if(playFromFileV){
-        if (!previewFileJack) return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
+        if (!previewFile) return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
         float* buffL=new float[nframes];
         float* buffR=new float[nframes];
         float* buff=new float[nframes*2];
-        long length=sfinfoForJackPreview.frames;
-        if(position>=0 && position+buffer_len<=length){
-            sf_readf_float(previewFileJack, buff,(sf_count_t)(nframes));
+        long length=sfinfoForPreview.frames;
+        if(position>=0 && position+buffer_len<length){
+            sf_readf_float(previewFile, buff,(sf_count_t)(nframes));
             int count=0;
             for (int i=0;i<nframes;i++){
                 buffL[i]=buff[count++];
@@ -395,58 +398,42 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
         zaparr(buffR);
         zaparr(buff);
     }else{
+        if(!soundManager || !soundManager->hasSound()) 
+            return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
         if(isPlaying()){
-                if(position>=rightLoop || position>=soundManager->getFrames()){
-                    position=leftLoop;
+            Q_ASSERT(position<soundManager->getFrames());
+            Q_ASSERT(position>=0);
+            bufferPointerL=&soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[position];
+            bufferPointerR=&soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[position];
+            if(position+nframes<getRightLoop()){
+                memcpy (p1, bufferPointerL, sizeof(jack_default_audio_sample_t) * nframes);
+                memcpy (p2, bufferPointerR, sizeof(jack_default_audio_sample_t) * nframes);
+                position+=nframes;
+            }else{
+                    memcpy (p1, bufferPointerL,
+                        sizeof(jack_default_audio_sample_t) * (getRightLoop()-position));
+                    memcpy (p2, bufferPointerR,
+                        sizeof(jack_default_audio_sample_t) * (getRightLoop()-position));
                     if(playingOnce){
-                        this->stopPlay();
+                        setJackNotPlaying(&(p1[getRightLoop()-position]),nframes-(getRightLoop()-position));
+                        setJackNotPlaying(&(p2[getRightLoop()-position]),nframes-(getRightLoop()-position));
+                        playing=FALSE;//this->stopPlay();
                         playingOnce=FALSE;
-                        return (setJackNotPlaying(p1,nframes) || setJackNotPlaying(p2,nframes));
-                    }
-                }
-                if(soundManager->isPlayingBandOnly()){
-                    float* outputLeft=new float[nframes];
-                    float* outputRight=new float[nframes];
-                    soundManager->getLeftChannelBand(outputLeft,position,position+nframes);
-                    soundManager->getRightChannelBand(outputRight,position,position+nframes);
-
-                    if (outputLeft) memcpy (p1, outputLeft, sizeof(jack_default_audio_sample_t) * nframes);
-                    else   setJackNotPlaying(p1,nframes);
-
-                    if (outputRight) memcpy (p2, outputRight, sizeof(jack_default_audio_sample_t) * nframes);
-                    else   setJackNotPlaying(p2,nframes);
-
-                    zaparr(outputLeft);
-                    zaparr(outputRight);
-                }else{
-                    bufferPointerL=
-                            &soundManager->getLeftChannel()[position];
-                    bufferPointerR=
-                            &soundManager->getRightChannel()[position];
-
-                    if(position+nframes<rightLoop){
-                        if (bufferPointerL) memcpy (p1, bufferPointerL, sizeof(jack_default_audio_sample_t) * nframes);
-                        else   setJackNotPlaying(p1,nframes);
-                        if (bufferPointerR) memcpy (p2, bufferPointerR, sizeof(jack_default_audio_sample_t) * nframes);
-                        else   setJackNotPlaying(p2,nframes);
-                        position+=nframes;
+                        position=leftLoop;
+                        positionChanged(position);
+                        resetLoop();
+                        return 0;
                     }else{
-                         if (bufferPointerL){
-                             memcpy (p1, bufferPointerL, sizeof(jack_default_audio_sample_t) * (rightLoop-position));
-                             memcpy (&p1[rightLoop-position],
-                                 &soundManager->getLeftChannel()[leftLoop],
-                                 sizeof(jack_default_audio_sample_t) * (nframes-(rightLoop-position)));
-                         }else   setJackNotPlaying(p1,nframes);
-                        if (bufferPointerR){
-                            memcpy (p2, bufferPointerL, sizeof(jack_default_audio_sample_t) * (rightLoop-position));
-                             memcpy (&p2[rightLoop-position],
-                                 &soundManager->getRightChannel()[leftLoop],
-                                 sizeof(jack_default_audio_sample_t) * (nframes-(rightLoop-position)));
-                        }else   setJackNotPlaying(p2,nframes);
-                        position=leftLoop+(nframes-(rightLoop-position));
+                        memcpy (&p1[getRightLoop()-position],
+                            &soundManager->getLeftChannel()[getLeftLoop()],
+                            sizeof(jack_default_audio_sample_t) * (nframes-(getRightLoop()-position)));
+                        memcpy (&p2[getRightLoop()-position],
+                            &soundManager->getRightChannel()[getLeftLoop()],
+                            sizeof(jack_default_audio_sample_t) * (nframes-(getRightLoop()-position)));
+                        position=getLeftLoop()+(nframes-(getRightLoop()-position));
                     }
-                }
-                positionChanged(position);
+            }
+            positionChanged(position);
         }else{
             setJackNotPlaying(p1,nframes);
             setJackNotPlaying(p2,nframes);
@@ -462,7 +449,7 @@ int SoundPlayer::jack_callback (jack_nframes_t nframes)
 void SoundPlayer::init()
 {
 
-    if(connect_jack() && connect_devdsp()){
+    if(connect_jack() && connect_portaudio() && connect_devdsp()){
         output=OUTPUT_NONE;
         std::cout<<"Oh, anyways we need no sound..Aren't we?\n";
     }
@@ -537,7 +524,9 @@ void SoundPlayer::playFromFile(QString path)
     playFromFileV=TRUE;
     playFromFilePath=path;
     //position=0;
-    if(output==OUTPUT_JACK)  previewFileJack= sf_open(path, SFM_READ, &sfinfoForJackPreview);
+    if(output==OUTPUT_JACK || output==OUTPUT_PORTAUDIO) 
+        previewFile= sf_open(path, SFM_READ, &sfinfoForPreview);
+    if(output==OUTPUT_PORTAUDIO) startPAStream();
 }
 
 /*!
@@ -578,7 +567,8 @@ void SoundPlayer::stopPlayFromFile()
     playing=FALSE;
     position=0;
     positionChanged(position);
-    if(output==OUTPUT_JACK) sf_close(previewFileJack);
+    if(output==OUTPUT_PORTAUDIO) stopPAStream();
+    if(output==OUTPUT_JACK || output==OUTPUT_PORTAUDIO) sf_close(previewFile);
 }
 
 /*!
@@ -587,7 +577,8 @@ void SoundPlayer::stopPlayFromFile()
 void SoundPlayer::pausePlayFromFile()
 {
     playFromFileV=FALSE;
-    //if(output==OUTPUT_JACK) sf_close(previewFileJack);
+    if(output==OUTPUT_PORTAUDIO) stopPAStream(); 
+    //if(output==OUTPUT_JACK) sf_close(previewFile);
 }
 
 
@@ -637,3 +628,191 @@ void SoundPlayer::removeSoundMonitor(QObject* sm)
 }
 
 
+/*!
+    \fn SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long framesPerBuffer,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags)
+ */
+int SoundPlayer::portaudio_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long nframes,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags)
+{
+    if(!soundManager) return paComplete;
+    if(playFromFileV){
+        if (!previewFile) return paComplete;
+        long length=sfinfoForPreview.frames;
+        //cout<<"position: "<<position<<endl;
+        if(position>=0 && position+nframes<length){
+            //cout<<"position: "<<position<<endl;
+            sf_readf_float(previewFile, (float*)outputBuffer,(sf_count_t)(nframes));
+            position+=nframes;
+            positionChanged(position);
+            return paContinue;
+        }else{
+            sf_readf_float(previewFile, (float*)outputBuffer,(sf_count_t)(length-position));
+            memset ((&((float*)outputBuffer)[length-position]), 0,
+                                 sizeof(float) * (nframes-(length-position))*2);
+            playFromFileV=FALSE;//stopPlayFromFile();
+            return paAbort;
+        }
+    }else{
+        if(isPlaying()){
+                if(position+nframes<getRightLoop()){
+                    soundManager->exportMultiplexedBuffer((float*)outputBuffer,
+                                            position,
+                                            position+nframes);
+                    position+=nframes;
+                }else{
+                    soundManager->exportMultiplexedBuffer((float*)outputBuffer,
+                                            position,
+                                            getRightLoop());
+                    if(playingOnce){
+                        memset ((&((float*)outputBuffer)[getRightLoop()-position]), 0,
+                                 sizeof(float) * (nframes-(getRightLoop()-position))*2);
+                        playing=FALSE; //stopPlay(); 
+                        playingOnce=FALSE;
+                        position=leftLoop;
+                        positionChanged(position);
+                        resetLoop(); 
+                        return paAbort;
+                    }else{
+                        soundManager->exportMultiplexedBuffer(
+                                                (&((float*)outputBuffer)[getRightLoop()-position]),
+                                                getLeftLoop(),
+                                                getLeftLoop()+(nframes-(getRightLoop()-position)));
+                        position=getLeftLoop()+(nframes-(getRightLoop()-position));
+                    }
+                }
+                positionChanged(position);
+                return paContinue;
+        }else{
+            return paComplete;
+        }
+    }
+    return paComplete;
+    
+}
+
+/*!
+    \fn SoundPlayer::portaudio_static_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long framesPerBuffer,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags, void *userData )
+ */
+int SoundPlayer::portaudio_static_callback( const void *inputBuffer, void *outputBuffer,
+                     unsigned long framesPerBuffer,
+                     const PaStreamCallbackTimeInfo* timeInfo,
+                     PaStreamCallbackFlags statusFlags, void *userData )
+{
+    return ((SoundPlayer *) userData)->portaudio_callback (inputBuffer,
+                                                           outputBuffer,
+                                                           framesPerBuffer,
+                                                           timeInfo,
+                                                           statusFlags);
+}
+
+
+/*!
+    \fn SoundPlayer::connect_portaudio()
+ */
+int SoundPlayer::connect_portaudio()
+{
+    std::cout<<"Querying Portaudio...\n";
+    PaError err=Pa_Initialize();
+    if( err != paNoError ){
+        printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        return 1;
+    }
+    
+    
+    
+    err = Pa_OpenDefaultStream(
+                    &portaudio_stream,        /* passes back stream pointer */
+                    0,              /* no input channels */
+                    2,              /* stereo output */
+                    paFloat32,      /* 32 bit floating point output */
+                    44100,          /* sample rate */
+                    256,            /* frames per buffer */
+                    portaudio_static_callback, /* specify our custom callback */
+                    (void *)this );        /* pass our data through to callback */
+    
+    if( err != paNoError ){
+        printf(  "PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        return 1;
+    }
+    output=OUTPUT_PORTAUDIO;
+    fprintf (stderr, "Connected to Portaudio.\n");
+    return 0;
+}
+
+
+/*!
+    \fn SoundPlayer::closePortaudio()
+ */
+void SoundPlayer::closePortaudio()
+{
+    Pa_CloseStream(portaudio_stream);
+    Pa_Terminate();
+}
+
+
+/*!
+    \fn SoundPlayer::startPAStream()
+ */
+int SoundPlayer::startPAStream()
+{
+    if(Pa_IsStreamActive(portaudio_stream)){
+        Pa_StopStream(portaudio_stream);
+    }
+    if(Pa_IsStreamStopped( portaudio_stream )==1){
+        PaError err=Pa_StartStream( portaudio_stream );
+        if( err != paNoError ){
+            printf(  "(in start) PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        }else{
+            //cout<<"Latency: "<<Pa_GetStreamInfo( portaudio_stream )->outputLatency<<"sec"<<endl;
+        }
+        return err;
+    } return 0;
+}
+
+
+/*!
+    \fn SoundPlayer::stopPAStream()
+ */
+int SoundPlayer::stopPAStream()
+{
+    if(Pa_IsStreamStopped( portaudio_stream )!=1){
+        PaError err=Pa_AbortStream( portaudio_stream );
+        if( err != paNoError ){
+            printf(  "(in stop) PortAudio error: %s\n", Pa_GetErrorText( err ) );
+        }else{
+            //while(portaudio_stream){}
+        }
+        return err;
+    }
+    return 0;
+}
+
+
+long SoundPlayer::getRightLoop() const {
+    if (position<rightLoop) return rightLoop;
+    else return soundManager->getFrames();
+}
+
+
+void SoundPlayer::setRightLoop(const long& theValue) {
+    rightLoop = theValue;
+}
+
+
+long SoundPlayer::getLeftLoop() const {
+    if (position<rightLoop) return leftLoop;
+    else return 0;
+}
+
+
+void SoundPlayer::setLeftLoop(const long& theValue) {
+    leftLoop = theValue;
+}
