@@ -18,6 +18,7 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include "wavecanvas.h"
+#include "rms.h"
 /**
         \fn WaveCanvas::WaveCanvas(QWidget* parent)
         Initialises member variables
@@ -39,15 +40,22 @@ WaveCanvas::WaveCanvas(QWidget* parent)
     hasPixmapV=FALSE;
     leftMaximaBuffer=0;
     rightMaximaBuffer=0;
+    leftRMSBuffer=0;
+    rightRMSBuffer=0;
     spectrogramMaxBuffer=0;
     spectrogramMaxBufferNbValues=0;
+    maxBufferNbValues=0;
     visibleWidth=0;
+    rmsL=rms_env_new();
+    rmsR=rms_env_new();
 }
 
 
 WaveCanvas::~WaveCanvas()
 {
     //zapBuffers();
+    rms_env_free(rmsL);
+    rms_env_free(rmsR);
 }
 
 /**
@@ -231,6 +239,8 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
     //cout<<"Drawing from "<<start_sample<<" to "<<end_sample<<"\n";
     if(soundManager!=0 && soundManager->hasSound()&&
        zoomRatio>=1 && width()>0 && height()>0){
+            QPointArray rmsPolygonL;
+            QPointArray rmsPolygonR;
             long jump=zoomRatio;
             if(jump<1) jump=1;
             if(start_sample<0)start_sample=0;
@@ -253,6 +263,11 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
 
 
             QColor wavecolor(50,50,255);
+            QColor rmscolor(255,0,0);
+            QPen wavepen(wavecolor,1);
+            QPen rmspen(rmscolor,1);
+            RasterOp waveRop=localpainter.rasterOp();
+            RasterOp rmsRop=Qt::AndROP;
             int translate=0;
 
             if(_hasRuler /*&& wasEmpty*/){
@@ -297,7 +312,7 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
             int x=c_x;
             //float sampleHeightFactor=(0.8/SHRT_MAX)*(v_height/4);
             float sampleHeightFactor=0.8*(v_height/4);
-            localpainter.setPen(wavecolor);
+            
             if(!leftMaximaBuffer || !rightMaximaBuffer){
                  //oh some caches..ya know..
                 leftMaximaBuffer=new maxAndMin[(int)ceil(soundManager->getFrames()/jump)];
@@ -306,16 +321,36 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
                     leftMaximaBuffer[i].hasValue=0;
                     rightMaximaBuffer[i].hasValue=0;
                 }
+                maxBufferNbValues=(int)ceil(soundManager->getFrames()/jump);
             }
+            rms_env_reset(rmsL);
+            rms_env_reset(rmsR);
+            bool rmsprefilled=FALSE;
             for (int i=start_sample;i<=end_sample-jump;i=i+jump){
                     float maxL=0;
                     float minL=0;
                     float maxR=0;
                     float minR=0;
+                    float maxRMSL=0;
+                    float maxRMSR=0;
+                    float maxRMSLbefore=0;
+                    float maxRMSRbefore=0;
                     if(!leftMaximaBuffer[i/zoomRatio].hasValue ||!rightMaximaBuffer[i/zoomRatio].hasValue){
+                        if(!rmsprefilled && start_sample>RMS_BUF_SIZE){
+                            //prefilling the rms buffer
+                            for (int i2=i-RMS_BUF_SIZE;i2<i;i2++){
+                                float valL=soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[i2];
+                                float valR=soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[i2];
+                                rms_env_process(rmsL,valL);
+                                rms_env_process(rmsR,valR);
+                            }
+                            rmsprefilled=TRUE;
+                        }
                         for (int j=i;j<=i+jump;j++){
                             float valL=soundManager->getLeftChannel(SoundHolder::FOR_OUTPUT)[j];
                             float valR=soundManager->getRightChannel(SoundHolder::FOR_OUTPUT)[j];
+                            float rmsvalL=rms_env_process(rmsL,valL);
+                            float rmsvalR=rms_env_process(rmsR,valR);
                             if(valL>=0){
                                 if(valL>maxL) maxL=valL;
                             }else if(valL<=0){
@@ -326,28 +361,82 @@ void WaveCanvas::drawWave(long start_sample,long end_sample)
                             }else if(valR<=0){
                                 if(valR<minR) minR=valR;
                             }
+                            
+                            if(rmsvalL>maxRMSL) maxRMSL=rmsvalL;
+                            if(rmsvalR>maxRMSR) maxRMSR=rmsvalR;
                         }
                         leftMaximaBuffer[i/jump].min=minL;
                         leftMaximaBuffer[i/jump].max=maxL;
+                        leftMaximaBuffer[i/jump].rms=maxRMSL;
                         leftMaximaBuffer[i/jump].hasValue=1;
+                        
                         rightMaximaBuffer[i/jump].min=minR;
                         rightMaximaBuffer[i/jump].max=maxR;
+                        rightMaximaBuffer[i/jump].rms=maxRMSR;
                         rightMaximaBuffer[i/jump].hasValue=1;
                     }else{
                         minL=leftMaximaBuffer[i/jump].min;
                         maxL=leftMaximaBuffer[i/jump].max;
+                        maxRMSL=leftMaximaBuffer[i/jump].rms;
+                        
                         minR=rightMaximaBuffer[i/jump].min;
                         maxR=rightMaximaBuffer[i/jump].max;
+                        maxRMSR=rightMaximaBuffer[i/jump].rms;
+                        
                     }
+                    maxRMSLbefore=i>0?leftMaximaBuffer[(i-1)/jump].rms:0;
+                    maxRMSRbefore=i>0?rightMaximaBuffer[(i-1)/jump].rms:0;
 
                     //if(hasPixmapV) return; //avoid cascading redraws
+                    localpainter.setPen(wavepen);
+                    //localpainter.setRasterOp(waveRop);
                     localpainter.drawLine(x,translate,x,(int)((float)-maxL*sampleHeightFactor)+translate);
                     localpainter.drawLine(x,translate,x,(int)((float)-minL*sampleHeightFactor)+translate);
 
                     localpainter.drawLine(x,translate+(double)v_height/2.0,
-                                                            x,(int)((float)-maxR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                                          x,(int)((float)-maxR*sampleHeightFactor)+translate+(double)v_height/2.0);
                     localpainter.drawLine(x,translate+(double)v_height/2.0,
-                                                            x,(int)((float)-minR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                                          x,(int)((float)-minR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                                          
+                    localpainter.setPen(rmspen);
+                    //localpainter.setRasterOp(rmsRop);
+                    if(x==0){
+                        localpainter.drawPoint(x,(int)((float)-maxRMSL*sampleHeightFactor)+translate);
+                        localpainter.drawPoint(x,(int)((float)-maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                        
+                        localpainter.drawPoint(x,(int)((float)+maxRMSL*sampleHeightFactor)+translate);
+                        localpainter.drawPoint(x,(int)((float)+maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0);
+                    }else{
+                         /*
+                        rmsPolygonL.putPoints(0,4,
+                                    x,(int)((float)-maxRMSL*sampleHeightFactor)+translate,
+                                    x-1,(int)((float)-maxRMSLbefore*sampleHeightFactor)+translate,
+                                    x,(int)((float)+maxRMSL*sampleHeightFactor)+translate,
+                                    x-1,(int)((float)+maxRMSLbefore*sampleHeightFactor)+translate);
+                                                    
+                        rmsPolygonR.putPoints(0,4,
+                                    x,(int)((float)-maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                    x-1,(int)((float)-maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                    x,(int)((float)+maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                    x-1,(int)((float)+maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0);
+                        
+                        
+                           localpainter.drawPolygon(rmsPolygonL,TRUE);
+                           localpainter.drawPolygon(rmsPolygonR,TRUE);
+                           */
+                            
+                        localpainter.drawLine(x,(int)((float)-maxRMSL*sampleHeightFactor)+translate,
+                                        x-1,(int)((float)-maxRMSLbefore*sampleHeightFactor)+translate);
+                        localpainter.drawLine(x,(int)((float)-maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                        x-1,(int)((float)-maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0);
+                        
+                        /*
+                        localpainter.drawLine(x,(int)((float)+maxRMSL*sampleHeightFactor)+translate,
+                                        x-1,(int)((float)+maxRMSLbefore*sampleHeightFactor)+translate);
+                        localpainter.drawLine(x,(int)((float)+maxRMSR*sampleHeightFactor)+translate+(double)v_height/2.0,
+                                        x-1,(int)((float)+maxRMSRbefore*sampleHeightFactor)+translate+(double)v_height/2.0);
+                          */
+                    }
 
                     x++;
                     /*
@@ -483,6 +572,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
                         max=FLT_MIN;
                         if(!spectrogramMaxBuffer[i/jump].hasValue){
                             for (int j=i;j<=i+jump;j=j+fftAdvance){
+                                        //cout<<"nbbands: "<<nbbands<<" j:"<<j<<" fa: "<<fftAdvance<<" band: "<<band<<endl;
                                         val=subband_energies[nbbands-1-band][j/fftAdvance];
                                         if(val>max) max=val;
                             }
@@ -496,8 +586,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
                             max=spectrogramMaxBuffer[i/jump].max[band];
                         }
 
-                        colorValue=MyMath::linear_regression(minimaLog,maximaLog,
-                                                                                                255,0,max);
+                        colorValue=MyMath::linear_regression(minimaLog,maximaLog,255,0,max);
                         colorValue=colorValue>255?255:colorValue<0?0:colorValue;
                         //color.setRgb(colorValue,colorValue,colorValue);
                         color.setHsv(colorValue,255,255);
@@ -543,7 +632,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
 
             }
             if(lastchunk>0){
-                localpainter.fillRect(x,0,x+lastchunk/zoomRatio,v_height,QBrush(blue));
+                localpainter.fillRect(x,0,v_width-c_x-x,v_height,QBrush(blue));
             }
             font.setPixelSize(12);
             font.setWeight(QFont::Bold);
@@ -586,7 +675,7 @@ void WaveCanvas::drawSpectrum(long start_sample,long end_sample)
 /*!
     \fn WaveCanvas::setPixmapXOrigin(int)
 
-    sets the origin of the pixmap
+    sets the origin of the pixmap. Used when repainting only the part of the image.
     \sa pixmapXOrigin
  */
 void WaveCanvas::setPixmapXOrigin(int x_orig)
@@ -614,6 +703,7 @@ void WaveCanvas::zapBuffers()
     //cout<<"in wavecanvas (parent ["<<parent()->parent()->name()<<"]) started zapping wave canvas buffers\n";
     if (leftMaximaBuffer) zaparr(leftMaximaBuffer);
     if (rightMaximaBuffer) zaparr(rightMaximaBuffer);
+
     if (spectrogramMaxBuffer){
         for (int i=0;i<spectrogramMaxBufferNbValues;i++){
             if (spectrogramMaxBuffer[i].max){
@@ -638,14 +728,34 @@ void WaveCanvas::zapBuffers(long start, long end)
         if (rightMaximaBuffer && &rightMaximaBuffer[i]) rightMaximaBuffer[i].hasValue=0;
         if (spectrogramMaxBuffer && &spectrogramMaxBuffer[i]) spectrogramMaxBuffer[i].hasValue=0;
     }
+    cout<<"in wavecanvas (parent ["<<parent()->parent()->name()<<"]) Buffers cleaned\n";
 }
 
 
 
 /*!
     \fn WaveCanvas::setVisibleWidth(int w)
+    Sets the with of the displayed part of the canvas.
  */
 void WaveCanvas::setVisibleWidth(int w)
 {
     visibleWidth=w;
 }
+
+
+/*!
+    \fn WaveCanvas::emptyBuffers()
+    Flushes the buffers containing max and min values for amplitude and spectrogram drawing.
+ */
+
+void WaveCanvas::emptyBuffers()
+{
+
+    for (int i=0;i<maxBufferNbValues;i++){
+        leftMaximaBuffer[i].hasValue=0;
+        rightMaximaBuffer[i].hasValue=0;
+    }
+    for (int i=0;i<spectrogramMaxBufferNbValues;i++){
+            spectrogramMaxBuffer[i].hasValue=FALSE;
+    }
+}
